

-- EPL_MATCH_TYPE


ALTER TABLE PPSNEPL.EPL_MATCH_TYPE
 DROP PRIMARY KEY CASCADE;

DROP TABLE PPSNEPL.EPL_MATCH_TYPE CASCADE CONSTRAINTS;


CREATE TABLE PPSNEPL.EPL_MATCH_TYPE
(
  MATCH_TYPE_ID      NUMBER(30)                 NOT NULL,
  MATCH_NAME         VARCHAR2(200 BYTE),
  MATCH_DESC         VARCHAR2(1000 BYTE),
  CREATED_BY         VARCHAR2(50 BYTE)          NOT NULL,
  CREATED_DTM        TIMESTAMP(6)               NOT NULL,
  LAST_MODIFIED_BY   VARCHAR2(50 BYTE),
  LAST_MODIFIED_DTM  TIMESTAMP(6)
);

GRANT SELECT ON PPSNEPL.EPL_MATCH_TYPE TO PPSNEPL_READ_ONLY_ROLE;

GRANT DELETE, INSERT, UPDATE ON PPSNEPL.EPL_MATCH_TYPE TO PPSNEPL_UPDATE_ROLE;

CREATE OR REPLACE PUBLIC SYNONYM EPL_MATCH_TYPE FOR PPSNEPL.EPL_MATCH_TYPE;


-- EPL_REMATCH_SUGGESTION TABLE


ALTER TABLE PPSNEPL.EPL_REMATCH_SUGGESTION
  DROP PRIMARY KEY CASCADE;

DROP TABLE PPSNEPL.EPL_REMATCH_SUGGESTION CASCADE CONSTRAINTS;

CREATE TABLE PPSNEPL.EPL_REMATCH_SUGGESTION
(
  EPL_REMATCH_ID    NUMBER(30)                  NOT NULL,
  TO_PRODUCT_ID_FK  NUMBER(30)                  NOT NULL
);

GRANT INSERT, UPDATE, DELETE ON PPSNEPL.EPL_REMATCH_SUGGESTION TO PPSNEPL_UPDATE_ROLE;

GRANT SELECT ON PPSNEPL.EPL_REMATCH_SUGGESTION TO PPSNEPL_READ_ONLY_ROLE;

CREATE OR REPLACE PUBLIC SYNONYM EPL_REMATCH_SUGGESTION FOR PPSNEPL.EPL_REMATCH_SUGGESTION;


-- EPL_REMATCH TABLE


ALTER TABLE PPSNEPL.EPL_REMATCH
 DROP PRIMARY KEY CASCADE;

DROP TABLE PPSNEPL.EPL_REMATCH CASCADE CONSTRAINTS;

CREATE TABLE PPSNEPL.EPL_REMATCH
(
  EPL_REMATCH_ID         NUMBER(30)             NOT NULL,
  FROM_PRODUCT_ID_FK     NUMBER(30)             NOT NULL,
  TO_PRODUCT_AUTO_ID_FK  NUMBER(30),
  MATCH_TYPE_ID_FK       NUMBER(30)             NOT NULL,
  NDF_UPDATE_FILE_FK     NUMBER(30),
  CREATED_BY             VARCHAR2(50)           NOT NULL,
  CREATED_DTM            TIMESTAMP(6)           NOT NULL,
  LAST_MODIFIED_BY       VARCHAR2(50),
  LAST_MODIFIED_DTM      TIMESTAMP(6)
);
    
GRANT INSERT, UPDATE, DELETE ON PPSNEPL.EPL_REMATCH TO PPSNEPL_UPDATE_ROLE;

GRANT SELECT ON PPSNEPL.EPL_REMATCH TO PPSNEPL_READ_ONLY_ROLE;

CREATE OR REPLACE PUBLIC SYNONYM EPL_REMATCH FOR PPSNEPL.EPL_REMATCH;


-- ALL RELATED REMATCH CONSTRAINTS 1) IX (IN CASE UNDERLYING A PK 2) PK 3) REF

CREATE UNIQUE INDEX PPSNEPL.MATCH_TYPE_IX
  ON PPSNEPL.EPL_MATCH_TYPE(MATCH_TYPE_ID);

CREATE UNIQUE INDEX PPSNEPL.REMATCH_IX
  ON PPSNEPL.EPL_REMATCH(EPL_REMATCH_ID);

CREATE INDEX PPSNEPL.MATCH_TYPE_ID_IX 
  ON PPSNEPL.EPL_REMATCH(MATCH_TYPE_ID_FK);

CREATE INDEX PPSNEPL.NDC_UPDATE_FILE_IX 
  ON PPSNEPL.EPL_REMATCH(NDF_UPDATE_FILE_FK);

ALTER TABLE PPSNEPL.EPL_MATCH_TYPE 
  ADD CONSTRAINT MATCH_TYPE_PK
  PRIMARY KEY (MATCH_TYPE_ID)
  USING INDEX PPSNEPL.MATCH_TYPE_IX
  ENABLE VALIDATE;

ALTER TABLE PPSNEPL.EPL_REMATCH 
  ADD CONSTRAINT EPL_REMATCH_PK
  PRIMARY KEY (EPL_REMATCH_ID)
  USING INDEX PPSNEPL.REMATCH_IX
  ENABLE VALIDATE;
    
ALTER TABLE PPSNEPL.EPL_REMATCH_SUGGESTION
  ADD CONSTRAINT EPL_REMATCH_SUGGESTION_PK
  PRIMARY KEY (EPL_REMATCH_ID, TO_PRODUCT_ID_FK)
  ENABLE VALIDATE;
    
ALTER TABLE PPSNEPL.EPL_REMATCH_SUGGESTION
 ADD CONSTRAINT EPL_REMATCH_SUGGESTION_R01 
  FOREIGN KEY (TO_PRODUCT_ID_FK) 
  REFERENCES PPSNEPL.EPL_PRODUCTS (EPL_ID)
  ENABLE VALIDATE;

ALTER TABLE PPSNEPL.EPL_REMATCH_SUGGESTION
 ADD CONSTRAINT EPL_REMATCH_SUGGESTION_R02 
  FOREIGN KEY (EPL_REMATCH_ID) 
  REFERENCES PPSNEPL.EPL_REMATCH (EPL_REMATCH_ID)
  ENABLE VALIDATE;

ALTER TABLE PPSNEPL.EPL_REMATCH ADD (
  CONSTRAINT EPL_REMATCH_MATCH_TYPE_R01 
  FOREIGN KEY (MATCH_TYPE_ID_FK) 
  REFERENCES PPSNEPL.EPL_MATCH_TYPE (MATCH_TYPE_ID)
  ENABLE VALIDATE);
  
ALTER TABLE PPSNEPL.EPL_REMATCH ADD (
  CONSTRAINT EPL_REMATCH_NDF_UPDATE_R02 
  FOREIGN KEY (NDF_UPDATE_FILE_FK) 
  REFERENCES PPSNEPL.EPL_NDF_UPDATE_FILE (NDF_UPDATE_FILE_ID)
  ENABLE VALIDATE);

ALTER TABLE PPSNEPL.EPL_REMATCH ADD (
  CONSTRAINT EPL_REMATCH_PRODUCTS_R03 
  FOREIGN KEY (FROM_PRODUCT_ID_FK) 
  REFERENCES PPSNEPL.EPL_PRODUCTS (EPL_ID)
  ENABLE VALIDATE);
  
ALTER TABLE PPSNEPL.EPL_REMATCH ADD (
  CONSTRAINT EPL_REMATCH_U01
  UNIQUE (FROM_PRODUCT_ID_FK, NDF_UPDATE_FILE_FK)
  ENABLE VALIDATE);


-- SEQUENCES


DROP SEQUENCE PPSNEPL.EPL_REMATCH_SEQ;

CREATE SEQUENCE PPSNEPL.EPL_REMATCH_SEQ
  START WITH 1
  MAXVALUE 9999999999999999999999999999
  MINVALUE 1
  NOCYCLE
  NOCACHE
  NOORDER;


CREATE OR REPLACE PUBLIC SYNONYM EPL_REMATCH_SEQ FOR PPSNEPL.EPL_REMATCH_SEQ;

GRANT ALTER, SELECT ON PPSNEPL.EPL_REMATCH_SEQ TO PPSNEPL_UPDATE_ROLE;