#!/bin/bash
SCRIPT=$0
SCRIPT_NAME=$(basename "${SCRIPT}")
LOG_FILE=/tmp/${SCRIPT_NAME}.log
exec > >(tee -a ${LOG_FILE} )
exec 2> >(tee -a ${LOG_FILE} >&2)
LOG_TIME=`date "+%Y-%m-%d %H:%M:%S"`

echo starting ${SCRIPT_NAME} at ${LOG_TIME} 

[ "`id -u -n`" = "oracle" ] || (echo "$0 must be run as oracle" && exit 1)

# Fail immediately on any error
set -e

#Common dir is one level above where this script lives, and allows us to find the other peer folders
export COMMON_DIR=$(dirname "${SCRIPT}")/..
export EXT_FILE_DIR=$COMMON_DIR/external-files
export FILES_TO_COPY_DIR=$COMMON_DIR/files-to-copy-oracle
export SCRIPTS_DIR=$COMMON_DIR/scripts

echo "Creating Oracle Database"	

$ORACLE_HOME/bin/dbca -silent -createDatabase -responseFile $SCRIPTS_DIR/dbca.rsp

#copy tnsnames.ora in, set owner and permissions.  
echo "Copying tnsnames.ora"	
cp $FILES_TO_COPY_DIR/tnsnames.ora $ORACLE_HOME/network/admin
chmod -R 644 $ORACLE_HOME/network/admin/tnsnames.ora

$ORACLE_HOME/bin/lsnrctl start LISTENER

echo "Setting up auto start and stop"	
mkdir -p /home/oracle/scripts
cp $FILES_TO_COPY_DIR/startup.sh /home/oracle/scripts/.
cp $FILES_TO_COPY_DIR/shutdown.sh /home/oracle/scripts/.
chmod u+x /home/oracle/scripts/startup.sh /home/oracle/scripts/shutdown.sh

echo "Creating tablespaces, profiles, roles, and accounts"
cd $SCRIPTS_DIR
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @drop_tablespaces
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @create_tablespaces
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @CREATE_PPSN_V3_PROFILES
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @CREATE_PPSN_V3_ROLES
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @CREATE_PPSN_V3_ACCOUNTS

echo "End of ${SCRIPT_NAME}"

