package gov.va.med.pharmacy.peps.common.utility;

import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;

/**
 * Utility methods for dealing with dates and times. All methods are static. This class should not be constructed.
 */
public final class DateTimeUtility {
  
  /**
   * This class should not be constructed.
   */
  private DateTimeUtility() {
  }

  /**
   * Converts the given {@link LocalDate} to an {@link Instant} in the system default timezone (i.e., {@link
   * ZoneId#systemDefault()}).
   * 
   * @param localDate the date to convert
   * @return an Instant at the start of the day
   */
  public static Instant toInstant(final LocalDate localDate) {
    // Implementation per <http://stackoverflow.com/q/23215299>.
    return localDate.atStartOfDay(ZoneId.systemDefault()).toInstant();
  }

}
