package gov.va.med.pharmacy.peps.common.utility;

import java.time.Instant;
import java.util.Objects;

/**
 * <p>
 * Utility class for performing conversions between Strings and other types common in PPS.
 * </p>
 * 
 * <p>
 * Probably other methods, such as most of {@link ReflectionUtility#convertStringValue(gov.va.med.pharmacy.peps.common.vo.FieldKey, String)}
 * should be moved to this class.
 * </p>
 */
public final class StringConverter {
    
    /**
     * This class should not be constructed.
     */
    private StringConverter() {
    }
    
    /**
     * Converts the given "milliseconds since epoch" string to an {@link Instant}. This conversion is useful because
     * dates are often persisted as these strings in PPS.
     * @param millisString numerical string representing milliseconds since epoch. Must not be null.
     * @return never null
     * @throws NumberFormatException if the string does not contain a parsable long
     */
    public static Instant millisToInstant(final String millisString) {
        Objects.requireNonNull(millisString);
        return Instant.ofEpochMilli(Long.parseLong(millisString));
    }

}
