package gov.va.med.pharmacy.peps.common.vista;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

/**
 * Represents a particular data element in VistA, including the file number, field number and IEN.
 */
public class VistaField {
    private String fileNumber;
    private String fieldNumber;
    private String ien;

    /**
     * Constructor.
     * 
     * @param fileNumber identifies the particular file in VistA
     * @param fieldNumber identifies the particular field within the top-level entity to which the newElements correspond, such
     *            as the effective date on a copay tier or the strength on an ingredient
     * @param ien identifies the top-level entity in VistA, such as the particular product, dosage form, etc.
     */
    public VistaField(String fileNumber, String fieldNumber, String ien) {
        this.fileNumber = fileNumber;
        this.fieldNumber = fieldNumber;
        this.ien = ien;
    }

    /**
     * Get the file number.
     * 
     * @return file number
     */
    public String getFileNumber() {
        return fileNumber;
    }

    /**
     * Get the field number.
     * 
     * @return field number
     */
    public String getFieldNumber() {
        return fieldNumber;
    }

    /**
     * Get the IEN.
     * 
     * @return ien
     */
    public String getIen() {
        return ien;
    }

    @Override
    public int hashCode() {
        return HashCodeBuilder.reflectionHashCode(this);
    }

    @Override
    public boolean equals(Object obj) {
        return EqualsBuilder.reflectionEquals(this, obj);
    }
}
