/**
 * Source file created in 2007 by Southwest Research Institute
 */
package gov.va.med.pharmacy.peps.common.vo;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.utility.NdfProductDataConversionUtility;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;

/**
 * Data representing a Product's Active Ingredient.
 */
public class ActiveIngredientVo extends ValueObject implements NdfSyncable {
    private static final long serialVersionUID = 1L;

    private IngredientVo ingredient;
    private DrugUnitVo drugUnit;
    private String strength;
    private boolean active;

    /**
     * getIngredient.
     * 
     * @return ingredient property
     */
    public IngredientVo getIngredient() {
        return ingredient;
    }

    /**
     * setIngredient.
     * 
     * @param ingredient ingredient property
     */
    public void setIngredient(IngredientVo ingredient) {
        this.ingredient = ingredient;
    }

    /**
     * getDrugUnit.
     * 
     * @return drugUnitVo property
     */
    public DrugUnitVo getDrugUnit() {
        return drugUnit;
    }

    /**
     * setDrugUnit.
     * 
     * @param drugUnitVo drugUnit property
     */
    public void setDrugUnit(DrugUnitVo drugUnitVo) {
        this.drugUnit = drugUnitVo;
    }

    /**
     * getStrength.
     * 
     * @return strength property
     */
    public String getStrength() {
        return strength;
    }

    /**
     * setStrength.
     * 
     * @param strength strength property
     */
    public void setStrength(String strength) {
        this.strength = trimToEmpty(strength);
    }

    /**
     * getActive.
     * 
     * @return activeYn property
     */
    public boolean getActive() {
        return active;
    }

    /**
     * setActive.
     * 
     * @param activeYn activeYn property
     */
    public void setActive(boolean activeYn) {
        this.active = activeYn;
    }

    /**
     * Returns the active ingredient ShortString. 
     * 
     * @return String properties and values of the ValueObject
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toShortString() {
        
        String s4 = (ingredient == null ? "" : ingredient.toShortString()) + " " 
            + (strength == null ? "" : strength) + " " + (drugUnit == null ? "" : drugUnit.toShortString()) 
            + "<br>";
      
        return s4;
    }

    @Override
    public List<SyncProcessorElement> asSyncProcessorElements(
        String vistaIen, 
        int pseudoSubfieldIen, 
        String userValue) {
        List<SyncProcessorElement> elements = new ArrayList<>();
        
        String ingredientIen = NdfProductDataConversionUtility.getStringValue(ingredient.getNdfIngredientIen());
        String vistaIenChild = vistaIen + "," + ingredientIen;

        elements.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                vistaIenChild, 
                ingredientIen, 
                userValue, 
                VistaProductFile.INGREDIENT_FIELD_NUMBER,
                SyncProcessorElement.IS_MULTIPLE_FIELD));
        
        elements.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                vistaIenChild, 
                NdfProductDataConversionUtility.getStringValue(strength), 
                userValue, 
                VistaProductFile.INGREDIENT_STRENGTH_FIELD_NUMBER, 
                SyncProcessorElement.IS_MULTIPLE_FIELD));
        
        elements.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                vistaIenChild, 
                NdfProductDataConversionUtility.convertDrugUnit(drugUnit), 
                userValue, 
                VistaProductFile.INGREDIENT_DRUG_UNIT_FIELD_NUMBER, 
                SyncProcessorElement.IS_MULTIPLE_FIELD));
        
        return elements;
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }
}
