package gov.va.med.pharmacy.peps.common.vo;

import java.lang.reflect.Field;

/**
 * Represents a site entry in the compliance report site list, with indicators whether it is should included or excluded.
 */
public class ComplianceSiteVo extends ValueObject {
    
    /** serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Indicator for when a site is part of the excluded list. */
    public static final String EXCLUDE = "E";
    
    /** Indicator for when a site is part of the included list. */
    public static final String INCLUDE = "I";
    
    /** The name. */
    private VistaSiteVo vistaSiteVo;

    /** Indicator whether the record is an include or an exclude. */
    private String includeExcludeIndicator;

    /**
     * Instantiates a new compliance site vo.
     *
     * @param vistaSiteVo the vista site vo
     * @param includeExcludeIndicator the include exclude indicator, using values {@link #INCLUDE} and {@link #EXCLUDE}
     */
    public ComplianceSiteVo(VistaSiteVo vistaSiteVo, String includeExcludeIndicator) {
        super();
        this.vistaSiteVo = vistaSiteVo;
        this.includeExcludeIndicator = includeExcludeIndicator;
    }

    /**
     * Empty Constructor.
     */
    public ComplianceSiteVo() {
        super();
    }

    /**
     * Gets the VistaSiteVo.
     *
     * @return the vistaSiteVo
     */
    public final VistaSiteVo getVistaSiteVo() {
        return vistaSiteVo;
    }

    /**
     * Sets the VistaSiteVo.
     *
     * @param vistaSiteVo the vistaSiteVo to set
     */
    public final void setVistaSiteVo(VistaSiteVo vistaSiteVo) {
        this.vistaSiteVo = vistaSiteVo;
    }

    /**
     * Gets the includeExcludeIndicator.
     *
     * @return the includeExcludeIndicator
     */
    public final String getIncludeExcludeIndicator() {
        return includeExcludeIndicator;
    }

    /**
     * Sets the includeExcludeIndicator.
     *
     * @param includeExcludeIndicator the includeExcludeIndicator to set
     */
    public final void setIncludeExcludeIndicator(String includeExcludeIndicator) {
        this.includeExcludeIndicator = includeExcludeIndicator;
    }

    /**
     * Convenience method to get the station name.
     *
     * @return the station name or "Station Name Not Found" if vistaSiteVo is null
     */
    public final String getStationName(){
        if (getVistaSiteVo() != null ){
            return getVistaSiteVo().getName();
        }
        return "Station Name Not Found";
    }
    
    
    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }    
}
