package gov.va.med.pharmacy.peps.common.vo;

import java.lang.reflect.Field;

/**
 * Represents a File-Based Site Entry, which contains enough information to do automatic filtering for the Compliance Report
 * site list.
 */
public class FileBasedSiteVo extends ValueObject {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 1L;

    /** The deactivation date. */
    private String deactivationDate;    
    
    /** The facility type code. */
    private String facilityTypeCode;        
    
    /** The name. */
    private String name;

    /** The station number. */
    private String stationNumber;


    /**
     * Constructor setting all properties.
     * @param deactivationDate Deactivation date of the site, a non-null value indicates it is inactive.
     * @param facilityTypeCode Facility Type Code - indicates the type of facility according to Vista/SDS Facility Types.
     * @param name Name of the Site
     * @param stationNumber Station Number of the site
     */
    public FileBasedSiteVo(String deactivationDate, String facilityTypeCode, String name, String stationNumber) {
        super();
        this.deactivationDate = deactivationDate;
        this.facilityTypeCode = facilityTypeCode;
        this.name = name;
        this.stationNumber = stationNumber;
    }

    /**
     * Gets the deactivation date.
     *
     * @return the deactivationDate
     */
    protected final String getDeactivationDate() {
        return deactivationDate;
    }

    /**
     * Sets the deactivation date.
     *
     * @param deactivationDate the deactivationDate to set
     */
    protected final void setDeactivationDate(String deactivationDate) {
        this.deactivationDate = deactivationDate;
    }

    /**
     * Gets the facility type code.
     *
     * @return the facilityTypeCode
     */
    protected final String getFacilityTypeCode() {
        return facilityTypeCode;
    }

    /**
     * Sets the facility type code.
     *
     * @param facilityTypeCode the facilityTypeCode to set
     */
    protected final void setFacilityTypeCode(String facilityTypeCode) {
        this.facilityTypeCode = facilityTypeCode;
    }

    /**
     * Gets the name.
     *
     * @return the name
     */
    protected final String getName() {
        return name;
    }

    /**
     * Sets the name.
     *
     * @param name the name to set
     */
    protected final void setName(String name) {
        this.name = name;
    }

    /**
     * Gets the station number.
     *
     * @return the stationNumber
     */
    protected final String getStationNumber() {
        return stationNumber;
    }

    /**
     * Sets the station number.
     *
     * @param stationNumber the stationNumber to set
     */
    protected final void setStationNumber(String stationNumber) {
        this.stationNumber = stationNumber;
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }    
}
