
package gov.va.med.pharmacy.peps.common.vo;


/**
 * Reasons for a rematch. For example, future inactivation.
 */
public enum MatchReason {

    /**
     * A product has been inactivated.
     */
    INACTIVATION("Inactivation", false),
    
    /**
     * A product has a proposed inactivation date.
     */
    FUTURE_INACTIVATION("Future Inactivation", false),
    
    /**
     * The user manually triggered a rematch.
     */
    MANUAL_REMATCH("MANUALLY-TRIGGERED REMATCH", false),
    
    /**
     * A product's VA Print Name has changed.
     */
    VA_PRINT_NAME("VA Print Name Change", true);

    private final String description;
    private final boolean defaultToSameProduct;

    private MatchReason(final String pDescription, final boolean pDefaultToSameProduct) {

        description = pDescription;
        defaultToSameProduct = pDefaultToSameProduct;
    }

    public String getDescription() {

        return description;
    }

    /**
     * Returns true if a rematch for this reason should default to the same product; false if it should not.
     * @return see description
     */
    public boolean getDefaultToSameProduct() {
        return defaultToSameProduct;
    }
    
}
