package gov.va.med.pharmacy.peps.common.vo;

import java.util.ArrayList;
import java.util.List;

/**
 * Interface for MigrationRow classes so generic logic can process various types of MigrationRows.
 */
public class MigrationRow {
    /**
     * Default Constructor.
     */
    public MigrationRow() {
        errors = new ArrayList<String>();
    }

    /** The errors. */
    private List<String> errors;

    /**
     * Checks if is valid.
     *
     * @return true, if is valid
     */
    public boolean isValid(){
        return errors.size() == 0;
    }

    /**
     * Adds a message describing a validation or processing error which occurred while processing the data for this row.
     * 
     * @param errorMessage to include in the error file for this row
     */
    public void addError(String errorMessage) {
        errors.add(errorMessage);
    }

    /**
     * Gets the error message.
     *
     * @return the error message
     */
    public String getErrorMessage() {
        return String.join(" ", errors);
    }

    /**
     * Returns "Y" or "N" to indicate whether an error occurred while processing this import row.
     * 
     * @return "Y" if an error occurred; otherwise, return "N".
     */
    public String getErrorFlag() {
        if (isValid()) {
            return "N";
        } else {
            return "Y";
        }
    };
    
    
}
