/**
 * Source file created in 2009 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo;


import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Objects;

import gov.va.med.pharmacy.peps.common.vo.validator.Errors;


/**
 * Contains data returned when a modification is submitted.
 */
public class ModificationSummaryVo extends ValueObject {
    private static final long serialVersionUID = 1L;

    private final ManagedItemVo unmodifiedItem;
    private Collection<ModDifferenceVo> modDifferences = new ArrayList<ModDifferenceVo>(0);
    private Errors warnings = new Errors();

    /**
     * Set the differences and warnings.
     * 
     * @param modDifferences Collection of {@link ModDifferenceVo}
     * @param warnings {@link Errors} object representing warnings
     */
    public ModificationSummaryVo(
        final ManagedItemVo unmodifiedItem, Collection<ModDifferenceVo> modDifferences, Errors warnings) {
        this.unmodifiedItem = Objects.requireNonNull(unmodifiedItem);
        this.modDifferences = modDifferences;
        this.warnings = warnings;
    }

    /**
     * Returns the original item (without any of the differences applied).
     * @return never null
     */
    public ManagedItemVo getUnmodifiedItem() {
      return unmodifiedItem;
    }

    /**
     * getModDifferences
     * 
     * @return modDifferences property
     */
    public Collection<ModDifferenceVo> getModDifferences() {
        return modDifferences;
    }

    /**
     * setModDifferences
     * 
     * @param modDifferences modDifferences property
     */
    public void setModDifferences(Collection<ModDifferenceVo> modDifferences) {
        this.modDifferences = new ArrayList<ModDifferenceVo>();

        if (modDifferences != null && !modDifferences.isEmpty()) {
            this.modDifferences.addAll(modDifferences);
        }
    }
    
    public boolean shouldTriggerRematch() {
      return !unmodifiedItem.determineRematchTriggers(ModDifferenceVo.extractDifferences(getModDifferences())).isEmpty();
    }

    /**
     * getWarnings for ModificationSummaryVO.
     * 
     * @return warnings property
     */
    public Errors getWarnings() {
        return warnings;
    }

    /**
     * setWarnings for ModificationSummaryVo.
     * 
     * @param warnings warnings property
     */
    public void setWarnings(Errors warnings) {
        this.warnings = warnings;
    }

    /**
     * Return true if the warnings is not null and if there are warnings for ModificationSummaryVO.
     * 
     * @return boolean
     */
    public boolean hasWarnings() {
        return warnings != null && !warnings.hasErrors();
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }
}
