/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo;

import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.I_INPATIENT;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.VISTA_INPATIENT_INDICATOR;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.VISTA_INPATIENT_OUTPATIENT_INDICATOR;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.VISTA_OUTPATIENT_INDICATOR;
import static gov.va.med.pharmacy.peps.common.vista.VistaFile.BLANK;

import java.lang.reflect.Field;
import java.util.Collection;

/**
 * Data Representing PossibleDosagesPackage
 */
public class PossibleDosagesPackageVo extends ValueObject implements Selectable {
    /**
     * Computes the the VistA package identifier from the given collection 
     * of PossibleDosagesPackageVo's.
     * 
     * @param packages to reduce
     * @return the VistA package identifier
     */
    public static String vistaPackageReduce(Collection<PossibleDosagesPackageVo> packages) {
        String newPackageValue = BLANK;
        
        if (packages.size() == 2) {
            newPackageValue = VISTA_INPATIENT_OUTPATIENT_INDICATOR;
        } else if (packages.size() == 1 
                && packages.iterator().next().getValue().contains(I_INPATIENT)) {
            newPackageValue = VISTA_INPATIENT_INDICATOR;
        } else if (packages.size() == 1) {
            newPackageValue = VISTA_OUTPATIENT_INDICATOR;
        }
        
        return newPackageValue;
    }
    
    private static final long serialVersionUID = 1L;
    private String value;

    /**
     * the constructor
     */
    public PossibleDosagesPackageVo() {
        super();
    }

    /**
     * getId for PossibleDosagesPackageVo.
     * 
     * @return id property
     * 
     * @see gov.va.med.pharmacy.peps.common.vo.Selectable#getId()
     */
    public String getId() {
        return value;
    }

    /**
     * getValue  for PossibleDosagesPackageVo.
     * 
     * @return value property
     * 
     * @see gov.va.med.pharmacy.peps.common.vo.Selectable#getValue()
     */
    public String getValue() {
        return value;
    }

    /**
     * setValue  for PossibleDosagesPackageVo.
     * 
     * @param value of String type
     */
    public void setValue(String value) {
        this.value = trimToEmpty(value);
    }

    /**
     * toShortString  for PossibleDosagesPackageVo.
     * 
     * @return short string value
     */
    @Override
    public String toShortString() {
        return value;
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }
}
