package gov.va.med.pharmacy.peps.common.vo;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.Date;

/**
 * This class represents a single row of data on the FDB Copay Change report.
 * 
 * <p>PPS-N 3.0 note: Does this class actually need to extend {@link ValueObject}? Maybe in order for it to be displayed
 * via a Print Template?</p>
 */
public class ReportFdbCopayChangeVo extends ValueObject {

    private static final long serialVersionUID = 1L;

    /**
     * The unique identifier of the PPS-N product.
     */
    private String id;
    
    /**
     * The VA Product Name.
     */
    private String vaProductName;
    
    /**
     * The orderable item name.
     */
    private String oiName;

    /**
     * The unique identifier of the parent orderable item.
     */
    private String parentId;    

    /**
     * The GCN sequence number.
     */
    private BigDecimal gcnSequenceNumber;
    
    /**
     * For inactive products, the date the product was
     * inactivated in PPS-N.
     */
    private Date inactivationDate;
    
    /**
     * The computed tier level of the product in the FDB
     * data.
     */
    private String fdbTierLevel;
    
    /**
     * The future tier level of the product in PPS-N.
     */
    private String futureTierLevel;
    
    /**
     * The federal legend code for the product in the FDB
     * data.
     */
    private String federalLegendCode;
    
    /**
     * The entity type.
     */
    private EntityType entityType;
    
    /**
     * The constructor.
     */
    public ReportFdbCopayChangeVo() {
        super();
        entityType = EntityType.PRODUCT;
    }
    
    /**
     * Get the VA product name.
     * 
     * @return the VA product name.
     */
    public final String getVaProductName() {
    
        return vaProductName;
    }
    
    /**
     * Set the VA product name.
     * 
     * @param vaProductName vaProductName
     */
    public final void setVaProductName(String vaProductName) {
    
        this.vaProductName = vaProductName;
    }
    
    /**
     * Get the orderable item name.
     * 
     * @return the orderable item name.
     */
    public final String getOiName() {
    
        return oiName;
    }
    
    /**
     * Set the orderable item name.
     * 
     * @param oiName oiName
     */
    public final void setOiName(String oiName) {
    
        this.oiName = oiName;
    }

    /**
     * Get the unique identifier of the parent Orderable Item.
     * 
     * @return the parent id.
     */
    public final String getParentId() {

        return parentId;
    }

    /**
     * Set the unique identifier of the parent Orderable Item.
     * 
     * @param parentId parentId
     */
    public final void setParentId(String parentId) {

        this.parentId = parentId;
    }

    /**
     * Get the GCN sequence number. It would be more idiomatic to store this as an {@link Integer}, but the database
     * Hibernate retrieves this as a {@link BigDecimal}.
     * 
     * @return the GCN sequence number.
     */
    public final BigDecimal getGcnSequenceNumber() {
    
        return gcnSequenceNumber;
    }
    
    /**
     * Set the GCN sequence number.
     * 
     * @param gcnSequenceNumber gcnSequenceNumber
     */
    public final void setGcnSequenceNumber(BigDecimal gcnSequenceNumber) {
    
        this.gcnSequenceNumber = gcnSequenceNumber;
    }
    
    /**
     * Get the inactivation date.
     * 
     * @return the inactivation date.
     */
    public final Date getInactivationDate() {
    
        return inactivationDate;
    }
    
    /**
     * Set the inactivation date.
     * 
     * @param inactivationDate inactivationDate
     */
    public final void setInactivationDate(Date inactivationDate) {
    
        this.inactivationDate = inactivationDate;
    }
    
    /**
     * Get the FDB tier level.
     * 
     * @return the FDB tier level.
     */
    public final String getFdbTierLevel() {
    
        return fdbTierLevel;
    }
    
    /**
     * Set the FDB tier level.
     * 
     * @param fdbTierLevel fdbTierLevel
     */
    public final void setFdbTierLevel(String fdbTierLevel) {
    
        this.fdbTierLevel = fdbTierLevel;
    }
    
    /**
     * Get the future tier level.
     * 
     * @return the future tier level.
     */
    public final String getFutureTierLevel() {
    
        return futureTierLevel;
    }
    
    /**
     * Set the future tier level.
     * 
     * @param futureTierLevel futureTierLevel
     */
    public final void setFutureTierLevel(String futureTierLevel) {
    
        this.futureTierLevel = futureTierLevel;
    }
    
    /**
     * Get the federal legend code.
     * 
     * @return the federal legend code.
     */
    public final String getFederalLegendCode() {
    
        return federalLegendCode;
    }
    
    /**
     * Set the federal legend code.
     * 
     * @param federalLegendCode federalLegendCode
     */
    public final void setFederalLegendCode(String federalLegendCode) {
    
        this.federalLegendCode = federalLegendCode;
    }

    /**
     * Get the id.
     * 
     * @return the id.
     */
    public final String getId() {

        return id;
    }

    /**
     * Set the id.
     * 
     * @param id id
     */
    public final void setId(String id) {

        this.id = id;
    }
    
    /**
     * Get the entity type.
     * 
     * @return the entity type.
     */
    public final EntityType getEntityType() {
    
        return entityType;
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }
    
}
