package gov.va.med.pharmacy.peps.domain.common.capability;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

import java.util.List;
import java.util.Optional;

/**
 * Performs CRUD operations on {@link RematchVo}s.
 */
public interface RematchMgtDomainCapability {
    
    /**
     * Updates the given rematch in the database.
     * @param rematch the rematch to update. May or may not have previously been saved to the database.
     * @param user the user updating it
     * @return the updated rematch from the database
     * @throws DuplicateRematchException if trying to create a rematch for a product when an unsent one already exists
     */
    public RematchVo saveOrUpdate(RematchVo rematch, UserVo user) throws DuplicateRematchException;
    
    /**
     * Deletes the given rematch from the database.
     * @param rematch the rematch to delete
     */
    public void delete(RematchVo rematch);

    /**
     * Returns a list of potential matches for a product. The returned products may be only minimally-populated.
     * 
     * @param productId the id of the product to match {@link ProductVo#getId()}
     * @return the list of potential matches in arbitrary order, can be zero or more.
     */
    public List<ProductVo> findRematchProducts(Long productId);

    /**
     * Returns the unsent rematch, if any, for the given product. (Only one unsent rematch may exist per product.)
     * @param productId the product's database ID
     * @return an Optional containing the rematch if one exists. Never null.
     */
    public Optional<RematchVo> getRematchForProduct(long productId);

}
