package gov.va.med.pharmacy.peps.domain.common.capability;


import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.vo.EplVistaUpdateVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaUpdateDo;

/**
 * <p>
 * Domain capability for {@link EplVistaUpdateVo}s.
 * </p>
 * 
 * <p>
 * Why does this interface not have any read methods? I'll tell you. At time of writing, these objects are read only from
 * {@link ReportDomainCapability}.
 * </p>
 */
public interface VistaUpdateCapability {

    /**
     * Saves the {@link EplVistaUpdateVo}. If it is from the National Site, it updates the file status and does not save a
     * EplVistaUpdate row.
     * 
     * @param eplVistaUpdateVo the update to save
     * @return the saved {@link EplVistaUpdateDo} if the update was for the national site and for the expected update file
     * @throws ValueObjectValidationException if any data in the given object was invalid. (Normally we would check this
     * in a validator, not in the domain capability layer, but this is a quick fix for now.)
     */
    public abstract EplVistaUpdateDo saveUpdate(EplVistaUpdateVo eplVistaUpdateVo) throws ValueObjectValidationException;

}
