package gov.va.med.pharmacy.peps.domain.common.capability.impl;


import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.EplVistaUpdateVo;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;
import gov.va.med.pharmacy.peps.domain.common.capability.VistaUpdateCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVistaUpdateDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaUpdateDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.EplVistaUpdateConverter;

import org.apache.logging.log4j.Logger; 
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Primary implementation of {@link VistaUpdateCapability}.
 */
@Component
public class VistaUpdateCapabilityImpl implements VistaUpdateCapability {
    private static final Logger LOG = LogManager.getLogger(VistaUpdateCapabilityImpl.class);

    private final EplVistaUpdateDao eplVistaUpdateDao;
    
    private final EplVistaUpdateConverter vistaUpdateConverter;
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pEplVistaUpdateDao The {@link EplVistaUpdateDao} to use.
     * @param pEplVistaUpdateConverter The {@link EplVistaUpdateConverter} to use.
     */
    @Autowired
    public VistaUpdateCapabilityImpl(
        final EplVistaUpdateDao pEplVistaUpdateDao, final EplVistaUpdateConverter pEplVistaUpdateConverter) {
      eplVistaUpdateDao = pEplVistaUpdateDao;
      vistaUpdateConverter = pEplVistaUpdateConverter;
    }

    @Override
    public EplVistaUpdateDo saveUpdate(EplVistaUpdateVo eplVistaUpdate) throws ValueObjectValidationException {
        LOG.debug("Received update from VistA: {}", new UntrustedLogParam(eplVistaUpdate));     
        
        EplVistaUpdateDo convertedDo;
        try {
            convertedDo = vistaUpdateConverter.convert(eplVistaUpdate);
        } catch (final IllegalArgumentException ex) {
            LOG.info("Received invalid data from VistA.", ex);
            // Right now we don't report specific errors to VistA so just pass in an empty Errors object.
            throw new ValueObjectValidationException(new Errors());
        }
        final EplVistaUpdateDo savedDo = eplVistaUpdateDao.save(convertedDo);  
        
        return savedDo;
    }
  
}
