package gov.va.med.pharmacy.peps.domain.common.dao;


import java.util.Date;
import java.util.List;
import java.util.Optional;

import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;

/**
 * Data Access Object for {@link EplRematchDo}s.
 */
public interface EplRematchDao extends DataAccessObject<EplRematchDo, Long>{

    /**
     * Returns the unsent rematch, if any, for the given product. (Only one unsent rematch may exist per product.)
     * 
     * @param productId the product's database ID
     * @return an Optional containing the rematch if one exists. Never null.
     */
    public Optional<EplRematchDo> retrieveForProduct(long productId);

    /**
     * Finds all automatic and suggested rematches that have not been sent in an update file. This will include future
     * and unapproved rematches.
     * 
     * @return the rematches. Never null.
     */
    public List<EplRematchDo> findUnsentRematches();

    /**
     * Finds the automatic and suggested rematches that are ready to send in an update file.
     * @param startDTM the start date of the reporting process requesting this data such that any newer data is not included 
     *
     * @return the rematches. Never null.
     */
    public List<EplRematchDo> findOutgoingRematches(Date startDTM);   

    public List<EplProductDo> findRematchProducts(long productId);

    public abstract void updateRematchNdfUpdateFileFk(Date startDTM, NdfUpdateFileVo ndfUpdateFileVo);

    public abstract void removeNdfUpdateFileFk(Long fileSeqNo);
    
}
