package gov.va.med.pharmacy.peps.domain.common.dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.hibernate.transform.Transformers;

import gov.va.med.pharmacy.peps.common.vo.ReportCopayTierChangeVo;
import gov.va.med.pharmacy.peps.common.vo.ReportFdbCopayChangeVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProdCopayTierAssocDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplProdCopayTierAssocDo;


/**
 * Primary implementation of {@link EplProdCopayTierAssocDao}.
 */
public class EplProdCopayTierAssocDaoImpl extends DataAccessObjectImpl<EplProdCopayTierAssocDo, Long>
    implements EplProdCopayTierAssocDao {
    
    /**
     * Finds an all EplProdCopayTierAssocDo object in ascending order from the database.
     * 
     * @return Collection of EplProdCopayTierAssocDo objects
    */
    @SuppressWarnings("unchecked")
    @Override
    public final List<EplProdCopayTierAssocDo> retrieve() {
        
        Session session = getCurrentSession();
        Criteria criteria = session.createCriteria(getDataObjectClass());
        Order order = Order.asc(EplProdCopayTierAssocDo.EPL_ID);
        order.ignoreCase();
        criteria.addOrder(order);
        return criteria.list();
    }
    
    /**
     * Return all DataObjects that have a field equal to the given value.
     * 
     * @param fieldName String name of the field to find by
     * @param value Object value of the field results must be equal to
     * @return List of DataObjects
     */
    @Override
    @SuppressWarnings("unchecked")
    public final List<EplProdCopayTierAssocDo> retrieve(String fieldName, Object value) {
        Criteria criteria = getCurrentSession().createCriteria(getDataObjectClass());

        criteria.add(Restrictions.eq(fieldName, value));
        Order order = Order.asc(EplProdCopayTierAssocDo.EPL_ID);
        order.ignoreCase();
        criteria.addOrder(order);

        return criteria.list();
    }
    
    @Override
    public final EplProdCopayTierAssocDo findById(Long id) {
        
        if(id == null){
            throw new IllegalArgumentException("EplCopayTier id cannot be null");
        }
        
        Session session = getCurrentSession();
        
        EplProdCopayTierAssocDo productCopayTier = (EplProdCopayTierAssocDo) session.get(EplProdCopayTierAssocDo.class, id);
        
        return productCopayTier;
    }
    
    
    /**
     * Save.
    */
    @Override
    public final EplProdCopayTierAssocDo save(EplProdCopayTierAssocDo eplProductCopayTier) {
        
        EplProdCopayTierAssocDo productCopayTier = new EplProdCopayTierAssocDo();
        
        Session session = getCurrentSession();
        
        Long id = (Long) session.save(eplProductCopayTier);
        
        productCopayTier = findById(id);
        
        return productCopayTier;
    }
    
    @Override
    @SuppressWarnings("unchecked")
    public final List<ReportFdbCopayChangeVo> retrieveFdbCopayChangeData(
        boolean includeInactiveProducts, 
        boolean includeNoGcnseqno,
        boolean includeFederalLegendCode) {
        
        //Convert booleans to ints for use in SQL.
        int intInactiveProducts = includeInactiveProducts ? 1 : 0;
        int intNoGcnseqno = includeNoGcnseqno ? 1 : 0;
        int intFederalLegendcode = includeFederalLegendCode ? 1 : 0;
        
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("fdbCopayChangeReport");
        query.setInteger("include_inactive_products", intInactiveProducts);
        query.setInteger("include_no_gcnseqno", intNoGcnseqno);
        query.setInteger("include_federal_legend_code", intFederalLegendcode);
        query.setResultTransformer(Transformers.aliasToBean(ReportFdbCopayChangeVo.class));
        List<ReportFdbCopayChangeVo> results = query.list();
        
        return results;
    }
    
    @Override    
    public final List<ReportCopayTierChangeVo> retrieveProductsWithCopayChangeData(
        Date startDate, 
        int includeActiveProducts, 
        int includeInactiveProducts, 
        String drugClassDelimiter) {
                
        java.sql.Date sqlDate = null;
        int ignoreStartDate = 1;
        
        if(startDate != null) {
            sqlDate = new java.sql.Date(startDate.getTime());
            ignoreStartDate = 0;
        }      
              
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("ppsCopayChangeReport");
        query.setDate("start_date", sqlDate);
        query.setInteger("include_active_products", includeActiveProducts);
        query.setInteger("include_inactive_products", includeInactiveProducts);
        query.setInteger("ignore_start_date", ignoreStartDate);
        query.setString("subfield_delimiter", drugClassDelimiter);
        query.setResultTransformer(Transformers.aliasToBean(ReportCopayTierChangeVo.class));
        
        @SuppressWarnings("unchecked")
        List<ReportCopayTierChangeVo> results = query.list();

        return results;
    }    
}
