/**
 * Source file created in 2006 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.domain.common.dao.impl;


import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Query;

import gov.va.med.pharmacy.peps.common.utility.StringConverter;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.dto.FutureInactiveProductDto;


/**
 * This class makes available the methods that may be used to access and 
 * modify the data referenced by the EplProductDao.
 */
public class EplProductDaoImpl extends DataAccessObjectImpl<EplProductDo, Long> implements EplProductDao {

    private static final String retrieveGncNdcMapSql = "SELECT prods.ndf_product_ien, prods.gcn_Seqno, ndc.ndc_No_Dashes " +
        "FROM Epl_Products prods JOIN (SELECT ndcs.epl_id_product_fk, ndcs.ndc_no_dashes, RANK() " +
        "OVER (PARTITION BY ndcs.epl_id_product_fk ORDER BY ndcs.ndc_no_dashes)rnk  " +
        "FROM epl_ndcs ndcs WHERE ndcs.item_status = 'ACTIVE') ndc ON prods.epl_id = ndc.epl_id_product_fk " +
        "AND ndc.rnk = 1 and prods.gcn_Seqno IS NOT NULL AND prods.item_Status = 'ACTIVE' " +
        "ORDER BY prods.ndf_product_ien";
    private static final String retrieveNdcProductNameSql =
        "SELECT prods.epl_id FROM Epl_Products prods, Epl_Ndcs ndcs where ndcs.ndc_no_dashes = ?" +
        " and epl_id_product_fk = prods.epl_id";

    @SuppressWarnings("unchecked")
    @Override
    public List<Object[]> retrieveInactive() {
        Query query = getCurrentSession().getNamedQuery("retrieveInactive");
        return (List<Object[]>) query.list();
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<Object[]> retrieveGcnNdcMap() {
        Query query = getCurrentSession().createSQLQuery(retrieveGncNdcMapSql);
        return query.list();
    }
    
  @SuppressWarnings("unchecked")
  @Override
  public List<EplProductDo> retrieveVaProduct(String ndc) {        
      Query query = getCurrentSession().createSQLQuery(retrieveNdcProductNameSql);
      query.setString(0, ndc);        
      return query.list();
  }
  
  @Override
  public List<FutureInactiveProductDto> findFutureInactive() {

    // NOTE WELL: These indicies must match the query select order.
    final int EPL_ID_INDEX = 0;
    final int VA_PRODUCT_NAME_INDEX = 1;
    final int CMOP_ID_INDEX = 2;
    final int FUTURE_DATE_INDEX = 3;

    @SuppressWarnings("unchecked") // Returns a List<Object[]> per Javadoc.
    final List<Object[]> rows = getCurrentSession().getNamedQuery("futureInactivationReport").list();
    
    // Transform the rows into DTO objects. We don't use the aliasToBean transformer here because we need the conversion
    // for setFutureDate().
    final List<FutureInactiveProductDto> dtos = new ArrayList<>(rows.size());
    for (final Object[] row : rows) {
        final FutureInactiveProductDto dto = new FutureInactiveProductDto();
        dto.setEplId((BigDecimal) row[EPL_ID_INDEX]);
        dto.setVaProductName((String) row[VA_PRODUCT_NAME_INDEX]);
        dto.setCmopId((String) row[CMOP_ID_INDEX]);
        dto.setFutureDate(StringConverter.millisToInstant((String) row[FUTURE_DATE_INDEX]));
        dtos.add(dto);
    }

    return dtos;
  }
}
