package gov.va.med.pharmacy.peps.domain.common.dao.impl;


import java.io.Serializable;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.object.VistaStatus;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.StatusVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNationalSettingDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfUpdateFileDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplStatusDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVistaUpdateDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplNationalSettingDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfUpdateFileDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplStatusDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaUpdateDo;


/**
 * Main implementation of EplVistaUpdateDao.
 * @author DNS   DNS
 */
public class EplVistaUpdateDaoImpl extends DataAccessObjectImpl<EplVistaUpdateDo, Long> implements EplVistaUpdateDao {
  
  private static final Logger LOG = LogManager.getLogger(EplVistaUpdateDaoImpl.class);

    @Autowired
    private SessionFactory sessionFactory;

    @Autowired
    private EplNdfUpdateFileDao eplNdfUpdateFileDao;
    @Autowired
    private EplStatusDao eplStatusDao;
    @Autowired
    private EplNationalSettingDao eplNationalSettingDao;

    @Override
    public EplVistaUpdateDo save(EplVistaUpdateDo eplVistaUpdate) {
              
        int nationalSite = populateNationalSite();
                    
        if (eplVistaUpdate.getSite() == nationalSite) {
            return saveNationalSiteUpdate(eplVistaUpdate);   
        }
        
        Session session = sessionFactory.getCurrentSession();        
        Serializable id = session.save(eplVistaUpdate);
        LOG.debug("Saved object with ID {}.", id);

        // This was not a national site update: return null per method specification.
        return null;
    }

    /**
     * Saves a status update from the National Site.
     * @param eplVistaUpdate
     * @return
     */
    private EplVistaUpdateDo saveNationalSiteUpdate(
            EplVistaUpdateDo eplVistaUpdate) {

        LOG.debug("Processing national VistA site update.");

        final EplStatusDo fileStatus = eplVistaUpdate.getEplNdfUpdateFileDo().getEplStatus();
        LOG.debug("Current file status is: {}", new UntrustedLogParam(fileStatus));
        if (!StatusVo.WAITING_FOR_NATIONAL_STATUSES.contains(fileStatus.getStatusId())) {
          // This is probably not the best thing to do: should return a validation error to VistA.
          LOG.warn("File status {} does not permit updates from national VistA.", fileStatus.getStatusId());
          return null;
        }
        EplNdfUpdateFileDo ndfUpdateFileDo = eplVistaUpdate.getEplNdfUpdateFileDo();
        ndfUpdateFileDo.setComments(eplVistaUpdate.getComments());
        if (VistaStatus.STARTED.toString().equals(eplVistaUpdate.getEplVistaStatus().getStatusName())) {
            ndfUpdateFileDo.setEplStatus(eplStatusDao.retrieve(StatusVo.START_OF_NATIONAL_PROCESSING));
        } 
        else if (VistaStatus.COMPLETED.toString().equals(eplVistaUpdate.getEplVistaStatus().getStatusName())) {
            eplVistaUpdate.getEplNdfUpdateFileDo().setEplStatus(eplStatusDao.retrieve(StatusVo.NATIONAL_INSTALL_SUCCESS));
        }
        else if (VistaStatus.ERROR.toString().equals(eplVistaUpdate.getEplVistaStatus().getStatusName())) {
            // if we receive a VistA error we automatically reject the file
            eplVistaUpdate.getEplNdfUpdateFileDo().setEplStatus(eplStatusDao.retrieve(StatusVo.VISTA_ERROR));
            eplVistaUpdate.getEplNdfUpdateFileDo()
                          .setComments("Automatically Rejected: Unable to Install Update File in the Test Account.");
        }
        else {
            eplVistaUpdate.getEplNdfUpdateFileDo().setEplStatus(eplStatusDao.retrieve(StatusVo.VISTA_ERROR));
            eplVistaUpdate.getEplNdfUpdateFileDo().setComments("Invalid Status Sent from VistA.");
        }
        eplNdfUpdateFileDao.update(eplVistaUpdate.getEplNdfUpdateFileDo(), new UserVo());
        return eplVistaUpdate;
    }

    /**
     * @return
     */
    private int populateNationalSite() {
        EplNationalSettingDo ns = eplNationalSettingDao.findByKeyName(NationalSetting.NATIONAL_SITE_NUMBER.toString());
        
        Long nationalSite = ns.getIntegerValue();
        return nationalSite.intValue();
    }

    public SessionFactory getSessionFactory() {
        return sessionFactory;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    /**
     * Gets the EplNdfUpdateFileDao.
     *
     * @return the eplNdfUpdateFileDao
     */
    public EplNdfUpdateFileDao getEplNdfUpdateFileDao() {
        return eplNdfUpdateFileDao;
    }

    /**
     * Sets the EplNdfUpdateFileDao.
     *
     * @param eplNdfUpdateFileDao the eplNdfUpdateFileDao to set
     */
    public void setEplNdfUpdateFileDao(EplNdfUpdateFileDao eplNdfUpdateFileDao) {
        this.eplNdfUpdateFileDao = eplNdfUpdateFileDao;
    }

    /**
     * Gets the EplStatusDao.
     *
     * @return the eplStatusDao
     */
    public EplStatusDao getEplStatusDao() {
        return eplStatusDao;
    }

    /**
     * Sets the EplStatusDao.
     *
     * @param eplStatusDao the eplStatusDao to set
     */
    public void setEplStatusDao(EplStatusDao eplStatusDao) {
        this.eplStatusDao = eplStatusDao;
    }

    /**
     * Gets the EplNationalSettingDao.
     *
     * @return the eplNationalSettingDao
     */
    public EplNationalSettingDao getEplNationalSettingDao() {
        return eplNationalSettingDao;
    }

    /**
     * Sets the EplNationalSettingDao.
     *
     * @param eplNationalSettingDao the eplNationalSettingDao to set
     */
    public void setEplNationalSettingDao(EplNationalSettingDao eplNationalSettingDao) {
        this.eplNationalSettingDao = eplNationalSettingDao;
    }

}
