package gov.va.med.pharmacy.peps.domain.common.model;


import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.MappedSuperclass;


/**
 * Base class for {@link DataObject}s that use Hibernate annotations instead of XML metadata.
 * 
 * @since 2.0
 */
@MappedSuperclass
public abstract class AnnotatedDataObject extends DataObject implements Serializable, Cloneable {

  private static final long serialVersionUID = 2L;

  /**
   * The standard number of digits allowed in the ID column. (Not all tables follow this standard at time of writing,
   * though.)
   */
  protected static final int ID_PRECISION = 30;

  /**
   * The maximum length of the {@link #getCreatedBy()} and {@link #getLastModifiedBy()} properties.
   */
  public static final int CREATED_BY_MAX_LENGTH = 50;

    // Override the getter just to add the JPA annotation.
    @Column(name = "CREATED_BY", nullable = false, length = CREATED_BY_MAX_LENGTH)
    @Override
    public String getCreatedBy() {
        return super.getCreatedBy();
    }

    // Override the getter just to add the JPA annotation.
    @Column(name = "CREATED_DTM", nullable = false)
    @Override
    public Date getCreatedDtm() {
        return super.getCreatedDtm();
    }

    // Override the getter just to add the JPA annotation.
    @Column(name = "LAST_MODIFIED_BY", length = CREATED_BY_MAX_LENGTH)
    @Override
    public String getLastModifiedBy() {
      return super.getLastModifiedBy();
    }

    // Override the getter just to add the JPA annotation.
    @Column(name = "LAST_MODIFIED_DTM")
    @Override
    public Date getLastModifiedDtm() {
      return super.getLastModifiedDtm();
    }
 
}
