package gov.va.med.pharmacy.peps.domain.common.model.dto;

import java.math.BigDecimal;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import gov.va.med.pharmacy.peps.common.vo.ProductReference;

/**
 * <p>
 * Data Transfer Object representing a change to a product.
 * </p>
 * <p>
 * Since this class is not used for updates, it would be better to make all fields final. But having setters allows us
 * to use Hibernate's aliasToBean transformers.
 * </p>
 */
public final class ProductChangeDto implements ProductReference {
    
    private Long eplId;
    private String vaProductName;
    private String cmopId;
    private String newValue;
    private String oldValue;
    
    /**
     * Default constructor.
     */
    public ProductChangeDto() {
    }
    
    /**
     * Constructs an instance specifying all fields.
     * 
     * @param eplId See {@link #setEplId(Long)}.
     * @param vaProductName See {@link #setVaProductName(String)}.
     * @param cmopId See {@link #setCmopId(String)}.
     * @param newValue See {@link #setNewValue(String)}.
     * @param oldValue See {@link #setOldValue(String)}.
     */
    public ProductChangeDto(Long eplId, String vaProductName, String cmopId, String newValue, String oldValue) {
        this.eplId = eplId;
        this.vaProductName = vaProductName;
        this.cmopId = cmopId;
        this.newValue = newValue;
        this.oldValue = oldValue;
    }

    /**
     * Returns the product's EPL ID.
     * @return possibly null
     */
    public Long getEplId() {
        return eplId;
    }
    public void setEplId(Long eplId) {
        this.eplId = eplId;
    }
    /**
     * See {@link #setNumericEplId(BigDecimal)}.
     * @return the EPL ID as a {@link BigDecimal}
     */
    public BigDecimal getNumericEplId() {
        return BigDecimal.valueOf(eplId);
    }
    /**
     * Sets the EPL ID as a BigDecimal to support native queries as well.
     * @param pEplId the value
     */
    public void setNumericEplId(final BigDecimal pEplId) {
        this.eplId = pEplId.longValue();
    }
    public String getVaProductName() {
        return vaProductName;
    }
    public void setVaProductName(String productName) {
        this.vaProductName = productName;
    }
    public String getCmopId() {
        return cmopId;
    }
    public void setCmopId(String cmopId) {
        this.cmopId = cmopId;
    }
    public String getNewValue() {
        return newValue;
    }
    public void setNewValue(String newName) {
        this.newValue = newName;
    }
    public String getOldValue() {
        return oldValue;
    }
    public void setOldValue(String oldName) {
        this.oldValue = oldName;
    }
    
    /**
     * Returns a string containing all of the object's public properties.
     */
    @Override
    public String toString() {
      return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
          .append(vaProductName)
          .append(cmopId)
          .append(newValue)
          .append(oldValue)
          .toString();
    }

}
