package gov.va.med.pharmacy.peps.domain.common.utility;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.va.med.pharmacy.peps.common.utility.EnvironmentUtility;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.RxNormClient;

import java.util.Date;
import java.util.List;

/**
 * RxNormCapability interface.
 * @author DNS
 *
 */
public interface RxNormCapability {

    /**
     * This method uses the product DAO to fetch a list of Product objects that are enabled for RxNorm update.
     * 
     * @return List
     */
    public abstract List<EplProductDo> fetchRxNormUpdateProducts();

    /**
     * This method attempts to retrieve RxNorm data for a product using calls to the NLM web service. For products for which the
     * web service return data, it populates the returned data into the product and uses the product DAO to persist the changes.
     * 
     * @param user UserVo property
     * @param product EplProductDo property
     * @param runDate Date property.
     */
    public abstract void populateAndPersist(UserVo user, EplProductDo product, Date runDate, ConfigFileUtility configFile);

    /**
     * Gets the rxNormClient property.
     * @return RxNormClient
     */
    public RxNormClient getRxNormClient();

    /**
     * Sets the rxNormClient property. 
     * @param rxNormClient RxNormClient property
     */
    public abstract void setRxNormClient(RxNormClient rxNormClient);

    /**
     * Gets the eplProductDao property.
     * @return EplProductDao
     */
    public EplProductDao getEplProductDao();

    /**
     * Sets the eplProductDao property.
     * @param eplProductDao EplProductDao property
     */
    public abstract void setEplProductDao(EplProductDao eplProductDao);

    /**
     * Gets the environmentUtility property.
     * @return EnvironmentUtility
     */
    public EnvironmentUtility getEnvironmentUtility();

    /**
     * Sets the environmentUtility property.
     * @param environmentUtility EnvironmentUtility property
     */
    public void setEnvironmentUtility(EnvironmentUtility environmentUtility);

    /**
     * Determines the differences between the product coding system properties 
     * and the current coding system ones.
     * @param code String property
     * @param product EplProductDo property
     * @return List
     */
    public List<Difference> getDifferences(String code, EplProductDo product);
}
