package gov.va.med.pharmacy.peps.service.common.update.impl;



import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ManufacturerVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.utility.update.item.ManufacturerConverter;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.common.ItemAction;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor;


/**
 * ManufacturerNdfFileSyncProcessor generates outgoing difference records for the NDF Update File for Manufacturer changes.
 */
public class ManufacturerNdfFileSyncProcessor extends AbstractSimpleNdfFileSyncProcessor implements NdfFileSyncProcessor {
    
    /** FieldKeys of the fields that, if modified, would be processed by the modification logic of this processor. */
    public static final Set<FieldKey> FIELDS = Collections.unmodifiableSet(new LinkedHashSet<FieldKey>(Arrays
        .asList(
            FieldKey.VALUE, 
            FieldKey.INACTIVATION_DATE,
            FieldKey.ITEM_STATUS
            )));    
    
    private static final String FILE_NUMBER = "55.95";    
    private static final String MANUFACTURER_IEN_FIELD_NUMBER = ".01";    
    private static final String INACTIVATION_DATE_FIELD_NUMBER = "2"; 
    
    /**
     * Instantiates a new ManufacturerNdfFileSyncProcessor.
     *
     * @param pNdfFileSyncCapability the ndf file sync capability
     * @see AbstractSimpleNdfFileSyncProcessor#AbstractSimpleNdfFileSyncProcessor(Set, String, String)
     */
    public ManufacturerNdfFileSyncProcessor(NdfFileSyncCapability pNdfFileSyncCapability) {
        super(FIELDS, FILE_NUMBER, MANUFACTURER_IEN_FIELD_NUMBER, pNdfFileSyncCapability);        
    }  

    @Override
    public void processNew(ManagedItemVo managedItem, UserVo user) {

        ManufacturerVo manufacturerVo = (ManufacturerVo) managedItem; 
        
        if (RequestItemStatus.APPROVED.equals(manufacturerVo.getRequestItemStatus()) && manufacturerVo.getValue() != null) {
            EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();

            diffObj.setVistaIen(manufacturerVo.getManufacturerIen());
            diffObj.setNewValue(manufacturerVo.getValue());  
            diffObj.setCreatedBy(user.getUsername());            
            processInsert(diffObj);             
        }

    }

    @Override
    public void processModified(ManagedItemVo managedItem, UserVo user, Collection<Difference> differences) {

        Map<FieldKey, Difference> setDifference = Difference.toDifferenceMap(differences);
        boolean hasDifference = ManufacturerConverter.hasNewOrModifiedFields(
            super.getFields(), setDifference, ItemAction.MODIFY);

        ManufacturerVo manufacturerVo = (ManufacturerVo) managedItem;

        if (hasDifference) {
            if (RequestItemStatus.APPROVED.equals(manufacturerVo.getRequestItemStatus()) && manufacturerVo.getValue() != null) {
                EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();

                diffObj.setVistaIen(manufacturerVo.getManufacturerIen());               
                diffObj.setVistaFieldNumber(INACTIVATION_DATE_FIELD_NUMBER);                
                diffObj.setNewValue(manufacturerVo.getInactivationDate() != null 
                    ? MumpsConverter.convertDate(manufacturerVo.getInactivationDate()) : VistaFile.BLANK);
                diffObj.setCreatedBy(user.getUsername());                
                processUpdate(diffObj); 
            }
        }

    }
   
    
}
