package gov.va.med.pharmacy.peps.domain.common.capability.impl;

import static org.easymock.EasyMock.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.easymock.EasyMockSupport;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.common.vo.ReportFdbCopayChangeVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProdCopayTierAssocDao;

/**
 * Automated unit tests for {@link ReportDomainCapabilityImpl}.
 */
public class ReportDomainCapabilityImplTest extends EasyMockSupport {

    @Test
    public void testGetFdbCopayChangeData() {
        /* Setup */

        // We want two changes for the same GCN, one with "O" and one with "F". Also include a change for a different
        // GCN with "F" and make sure it is preserved.
        final ReportFdbCopayChangeVo copayChange1 = new ReportFdbCopayChangeVo();
        copayChange1.setId("1");
        copayChange1.setVaProductName("Product 1");
        copayChange1.setOiName("OI 1");
        copayChange1.setGcnSequenceNumber(new BigDecimal(100));
        copayChange1.setInactivationDate(null);
        copayChange1.setFdbTierLevel("2");
        copayChange1.setFutureTierLevel("3");
        copayChange1.setFederalLegendCode("O");
        final ReportFdbCopayChangeVo copayChange2 = new ReportFdbCopayChangeVo();
        copayChange2.setId("2");
        copayChange2.setVaProductName("Product 2");
        copayChange2.setOiName(copayChange1.getOiName());
        copayChange2.setGcnSequenceNumber(copayChange1.getGcnSequenceNumber());
        copayChange2.setInactivationDate(null);
        copayChange2.setFdbTierLevel("3");
        copayChange2.setFutureTierLevel("3");
        copayChange2.setFederalLegendCode("F");
        final ReportFdbCopayChangeVo copayChange3 = new ReportFdbCopayChangeVo();
        copayChange3.setId("3");
        copayChange3.setVaProductName("Product 3");
        copayChange3.setOiName("OI 3");
        copayChange3.setGcnSequenceNumber(new BigDecimal(200));
        copayChange3.setInactivationDate(new Date());  // Include an inactivation date just for kicks.
        copayChange3.setFdbTierLevel("1");
        copayChange3.setFutureTierLevel("2");
        copayChange3.setFederalLegendCode("F");
        
        final EplProdCopayTierAssocDao mockCopayTierAssocDao = strictMock(EplProdCopayTierAssocDao.class);
        final EnvironmentUtilityStub mockEnvironmentUtility = new EnvironmentUtilityStub();
        mockEnvironmentUtility.setNational();

        final ReportDomainCapabilityImpl objUnderTest = new ReportDomainCapabilityImpl(
            // We can get away with a null DataSource for this test. It is unused.
            null, mockCopayTierAssocDao, mockEnvironmentUtility);
        
        expect(mockCopayTierAssocDao.retrieveFdbCopayChangeData(false, false, true))
            .andReturn(Arrays.asList(copayChange1, copayChange2, copayChange3));
        
        replayAll();
        
        /* Behavior & Verification */

        final List<ReportFdbCopayChangeVo> actualList = objUnderTest.getFdbCopayChangeData(false, false, true);
        assertThat(actualList, equalTo(Arrays.asList(copayChange1, copayChange3)));
    }

}
