package gov.va.med.pharmacy.peps.domain.common.dao.impl.test;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;


import java.util.Date;
import org.apache.logging.log4j.Logger;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;


import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.impl.EplIngredientDaoImpl;
import gov.va.med.pharmacy.peps.domain.common.model.EplIngredientDo;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;

/**
 * Automated tests for {@link EplIngredientDaoImpl}.
 *
 */
public class EplIngredientDaoImplIT extends AbstractDomainIT{
  
  private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(
      EplIngredientDaoImplIT.class);
  
  @Autowired
  private EplIngredientDaoImpl eplIngredientDao;
  
  
  /**
   * Tests that {@link EplIngredientDaoImpl#findIngredientByName(String)} returns the expected data.
   */
  @Test
  @Transactional
  public void testFindIngredientByName() {
    
    /* Test Parameters */
    final Date testDate = new Date();
    
    /* Setup */
    String username = "AutomatedTest";
    final UserVo user = new UserVo();
    user.setUsername(username);
    final EplIngredientDo ingredientToDatabase = new EplIngredientDo();
    ingredientToDatabase.setName("MixedCase");
    // Using unrealistic values in order to avoid any conflicts with the valid data                                                                                                                  
    ingredientToDatabase.setEplId(999999L);
    ingredientToDatabase.setVuid("TestVuid");
    ingredientToDatabase.setMasterEntryForVuid("Z");
    ingredientToDatabase.setRequestStatus("TestRequestStatus");
    ingredientToDatabase.setItemStatus("TestItemStatus");
    ingredientToDatabase.setRevisionNumber(999999L);
    ingredientToDatabase.setCreatedBy(username);
    ingredientToDatabase.setCreatedDtm(testDate);
    
    LOG.debug("Saving ingredient to database");
    eplIngredientDao.insert(ingredientToDatabase, user);
    
    /* Behavior */
    LOG.debug("Getting ingredient from database");
    final EplIngredientDo ingredientFromDatabase = eplIngredientDao.findIngredientByName("mIXEDcASE");
    
    /* Verification */
    assertThat(ingredientToDatabase, equalTo(ingredientFromDatabase));
  }

}
