package gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.test;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.junit4.rules.SpringClassRule;
import org.springframework.test.context.junit4.rules.SpringMethodRule;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.RxNormClient;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.RxNormData;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;

import java.util.Arrays;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Assert;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;

/**
 * This class basically tests the calls to the ISAAC service to get rxnorm codes.
 */
@RunWith(Parameterized.class)
public class RxNormClientIT extends AbstractDomainIT {

    private static final Logger LOG = LogManager.getLogger(RxNormClientIT.class);
    
    /**
     * Enables Spring Test integration without the runner.
     */
    @ClassRule
    public static final SpringClassRule SPRING_CLASS_RULE = new SpringClassRule();
    
    /**
     * Enables Spring Test integration without the runner.
     */
    @Rule
    public final SpringMethodRule methodRule = new SpringMethodRule();
    
    @Autowired
    private RxNormClient rxnormClient;

    private final String vuid;

    private static final List<String> VUIDS = Arrays.asList(
        "4001164", "4005733", "4002412", "4003547", "4007158", "4007183", "4001954");
    
    /**
     *Run each test for each VUID.
     * 
     * @return list of VUID parameters 
     */
    @Parameters
    public static List<String> data() {
        return VUIDS;
    }
    
    /**
     *Constructs instance with test parameters.
     *
     * @param pVuid
     */
    public RxNormClientIT(final String pVuid) {
        vuid = pVuid;
    }

    /**
     * Prepares the class for testing.
     */
    @Before
    public void setup() throws Exception {
        EnvironmentUtilityStub environmentUtility = new EnvironmentUtilityStub();
        environmentUtility.setNational();
    }

    /**
     * Test the calls to the ISAAC service to get some codes.
     */
    @Test
    public void testGetRxNormCodeFromIsaac() {

        String code = rxnormClient.getCode(vuid, new ConfigFileUtility());
        LOG.info(code);
        Assert.assertNotNull("should receive code back", code);
    }

    /**
     * Test the calls to the ISAAC service to get RX Norm Data.
     */
    @Test
    public void testGetRxNormDataFromIsaac() {
        RxNormData data = rxnormClient.getRxNormData(vuid, new ConfigFileUtility());
        LOG.info(data);
        Assert.assertNotNull("should receive RxNormData back", data);
    }

}
