package gov.va.med.pharmacy.peps.domain.common.utility.converter;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.UserVo;

/**
 * Automated unit tests for {@link RequestConverter}.
 */
public class RequestConverterTest {

    /**
     * Tests {@link RequestConverter#fromUser(gov.va.med.pharmacy.peps.common.vo.UserVo)} and
     * {@link RequestConverter#toUser(String)}.
     */
    @SuppressWarnings("deprecation")  // Intentionally testing a deprecated method here.
    @Test
    public void testFromToUser() {
        /* Setup */
        final UserVo sampleUser = new UserVo();
        sampleUser.setFirstName("FirstName");
        sampleUser.setLastName("LastName");
        sampleUser.setUsername("UserName");
        sampleUser.setLocation("Location");

        final RequestConverter objUnderTest = new RequestConverter();

        /* Behavior & Verification */
        final String userString = objUnderTest.fromUser(sampleUser);
        
        assertThat(RequestConverter.toUser(userString), equalTo(sampleUser));
        // Sanity check: make sure UserVo.equals() is working properly.
        assertThat(RequestConverter.toUser("Some_Random_String"), not(equalTo(sampleUser)));
    }
    
    /**
     * Verifies that a username with an underscore results in an exception.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testFromUser_Illegal() {
        final UserVo sampleUser = new UserVo();
        sampleUser.setFirstName("FirstName");
        sampleUser.setLastName("LastName");
        sampleUser.setUsername("User_Name");
        sampleUser.setLocation("Location");

        final RequestConverter objUnderTest = new RequestConverter();

        /* Behavior & Verification */
        objUnderTest.fromUser(sampleUser);
    }

}
