/**
 * 
 */
package gov.va.med.pharmacy.peps.domain.common.utility.test;

import java.util.Date;

import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfUpdateFileDo;

/**
 * Creates an EplNdfOutgoingDifferences with the specified values, entering test values for other unspecified fields, defaulting
 * the file FK to null.
 *
 */
public final class EplNdfOutgoingDifferencesGenerator {

  /**
   * Private constructor since this class just contains static methods. 
   */
  private EplNdfOutgoingDifferencesGenerator() {
  }

  /**
   * Creates an EplNdfOutgoingDifferences with the specified values, entering test values for other unspecified fields.
   *
   * @param fileNumber the file number
   * @param fieldNumber the field number
   * @param ien the ien
   * @param action the action
   * @return the epl ndf outgoing differences
   */
  public static EplNdfOutgoingDifferences createDiff(final String fileNumber, final String fieldNumber, final String ien,
      final String action) {
    return createDiff(fileNumber, fieldNumber, ien, action, null);
  }

  /**
   * Creates an EplNdfOutgoingDifferences with the specified values, entering test values for other unspecified fields.
   *
   * @param fileNumber the file number
   * @param fieldNumber the field number
   * @param ien the ien
   * @param action the action
   * @param ndcUpdateFileFk the ndc update file fk
   * @return the epl ndf outgoing differences
   */
  public static EplNdfOutgoingDifferences createDiff(final String fileNumber, final String fieldNumber, final String ien,
      final String action, EplNdfUpdateFileDo ndcUpdateFileFk) {
    final EplNdfOutgoingDifferences diff = new EplNdfOutgoingDifferences();
    diff.setActionType(action);
    diff.setCreatedBy("Automated Test");
    diff.setCreatedDtm(new Date());
    diff.setNewValue("NewValue");
    diff.setVistaFileNumber(fileNumber);
    diff.setVistaFieldNumber(fieldNumber);
    diff.setVistaIen(ien);
    diff.setNdcUpdateFileFk(ndcUpdateFileFk);

    return diff;
  }
}
