package gov.va.med.pharmacy.peps.service.common.update.impl;

import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;

/**
 * Helper class for JUnit tests which need to calculate checksums for use
 * with the NDF update file consolidation logic.
 */
public abstract class ChecksumCalculator {
    /**
     * Calculate a checksum for the given list of CopayTierAssocVo's.
     * @param copayTierAssocs to use in the calculation
     * @return checksum representing the given list
     */
    public static String forCopayTierAssocVos(
        List<CopayTierAssocVo> copayTierAssocs) {
        List<String> values = new ArrayList<String>();
        
        for(CopayTierAssocVo vo : copayTierAssocs) {
            values.add(vo.getCopayTier().getTierLevel());
            values.add(MumpsConverter.convertDate(vo.getEffectiveDate()));
            
            if(vo.getEndDate() != null) {
                values.add(MumpsConverter.convertDate(vo.getEndDate()));
            }
        }
        
        String result = String.join("~", values);
        
        if(result.isEmpty()) {
            result = null;
        }
        
        return result;
    }
}
