/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.test;


import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.external.common.document.ExceptionDocument;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.document.NdcItemDocument;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.document.OrderableItemDocument;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.item.ndcitem.NdcItem;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.item.orderableitem.OrderableItem;
import junit.framework.TestCase;


/**
 * Tests the example XML documents.
 * 
 * 
 */
public class XmlDocumentIT extends TestCase {

    private static final String NOTNULL = "item shouldn't be null!";
    
    /**
     * setUp
     * @throws Exception exception
     * 
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
    }

    /**
     * Test NDC Item.
     * 
     * @throws Exception exception
     */
    public void testNdcItemDocument() throws Exception {
        NdcItem item = NdcItemDocument.instance().unmarshal(readDocument(new File("etc/xml/document/item/ndcItem.xml")));
        assertNotNull(NOTNULL, item);

        String xml = NdcItemDocument.instance().marshal(item);

        assertFalse("UPN should not be nil", NdcItemDocument.instance().unmarshal(xml).getNdcUpnFile().getUpn().isNil());
    }

    /**
     * Test nullable NDC Item.
     * 
     * @throws Exception exception
     */
    public void testNullableNdcItemDocument() throws Exception {
        NdcItem item = NdcItemDocument.instance().unmarshal(
            readDocument(new File("etc/xml/document/item/ndcItem-Nullable.xml")));
        assertNotNull(NOTNULL, item);

        // (mspears:Nov 17, 2008) Defect in JAXB - https://jaxb.dev.java.net/issues/show_bug.cgi?id=565
        //      (jbarde: 29 Mar 2012) - defect fixed as of JAXB 2.1.10, which is used currently. 
        item.getNdcUpnFile().getOtxRxIndicator().setNil(false);
        item.getNdcUpnFile().getPackageSize().setNil(false);
        item.getNdcUpnFile().getPackageType().setNil(false);
        item.getNdcUpnFile().getTradeName().setNil(false);

        String xml = NdcItemDocument.instance().marshal(item);
        assertTrue("UPN should be nil", NdcItemDocument.instance().unmarshal(xml).getNdcUpnFile().getUpn().isNil());
    }

    /**
     * Test Orderable Item.
     * 
     * @throws Exception exception
     */
    public void testOrderableItemDocument() throws Exception {
        OrderableItem item = OrderableItemDocument.instance().unmarshal(
            readDocument(new File("etc/xml/document/item/orderableItem.xml")));
        assertNotNull(NOTNULL, item);

        String xml = OrderableItemDocument.instance().marshal(item);
        OrderableItemDocument.instance().unmarshal(xml);
    }

    /**
     * Test Exception.
     * 
     * @throws Exception exception
     */
    public void testExceptionDocument() throws Exception {
        gov.va.med.pharmacy.peps.external.common.preencapsulation.vo.exception.Exception item = ExceptionDocument.instance()
            .unmarshal(readDocument(new File("etc/xml/document/status/exception.xml")));
        assertNotNull(NOTNULL, item);

        String xml = ExceptionDocument.instance().marshal(item);

        ExceptionDocument.instance().unmarshal(xml);
    }

    /**
     * Reads the specified file into a single String.
     * 
     * @param file file to read
     * @return the file contents
     * @throws IOException exception
     */
    protected static String readDocument(File file) throws IOException {
        InputStream in = new FileInputStream(file);
        ByteArrayOutputStream out = new ByteArrayOutputStream();

        // Read XML document for the XmlDocumentTest
        try {
            byte[] buffer = new byte[PPSConstants.I1024];

            for (int length = in.read(buffer); length > 0; length = in.read(buffer)) {
                out.write(buffer, 0, length);
            }
        } finally {
            in.close();
            out.close();
        }

        // return the Read XML document for the XmlDocumentTest
        return new String(out.toByteArray());
    }
}
