package gov.va.med.pharmacy.peps.common.utility.security;

import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.List;

/**
 * The StdJavaListEnum is used in conjunction with the VoListEnum to create a white list of safe classes used by
 * SafeObjectInputStream for safe deserialization of Objects. StdJavaListEnum contains Java standard classes that are used by
 * ValueObject classes that may be deserialized.
 * 
 * As ValueObject classes are added to the code, the classes associated with the new ValueObject class need to be added to the
 * safe classes list. TheVoListEnumTest is used to walk through all of the classes within the PS_PPS_common project to determine
 * which classes need to be added to the enumerations as a safe class. For classes indicated as missing from the safe list by
 * VoListEnumTest, Pharmacy classes should be added to the VoListEnum. Other Java classes should be added to this
 * StdJavaListEnum.
 * 
 * The logic within SafeObjectInputStream to check for safe classes specifically calls out the Array, Enum, and List classes. In
 * order to allow some inner classes to be verified as safe for deserialization, these classes needed to be checked differently
 * then simply locating them on this StdJavaListEnum. These classes do not need to be part of this enumeration, but are included
 * for completeness.
 */
public enum StdJavaListEnum {

    /** The stdjavalist. */
    STDJAVALIST;

    private static Class<?>[] StdJavaList = {
        // Standard Java Classes
        Array.class,
        Enum.class,
        Object.class,
        Class.class,
        String.class,
        Number.class,
        Double.class,
        Long.class,
        Integer.class,
        Boolean.class,

        // Java Utils Classes 
        List.class,
        java.sql.Timestamp.class,
        java.util.ArrayList.class,
        java.util.Map.class,
        java.util.HashMap.class,
        java.util.LinkedHashMap.class,
        java.util.Date.class,
        java.util.Collection.class,
        java.util.HashSet.class,
        java.util.Set.class,
        java.text.SimpleDateFormat.class,
        java.math.BigDecimal.class
    };

    /**
     * Provide the enumeration as a list of classes. 
     * @return list of classes 
     */
    public static List<Class<?>> getStdJavaList() {
        return Arrays.asList(StdJavaList);
    };

}

