package gov.va.med.pharmacy.peps.service.common.capability;

import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.ProductVo;

/**
 * Bean used to store information needed to prepare a rematch to be committed.
 */
public class RematchPreperationBean {
    
    private final boolean rematchExistsForProduct;
    private final List<ProductVo> potentialRematchProducts;
    private final boolean defaultToFirstProduct;

    /**
     * Constructor the object with the required parameters.
     * @param pRematchExistsForProduct whether a rematch already exists for the given rematch's from product.
     * @param pPotentialRematchProducts the list of potential rematches for the given rematch's from product.
     * @param pDefaultToFirstProduct See {@link #getDefaultToFirstProduct()}. Must only be true if the potential products
     * list is non-empty.
     */
    public RematchPreperationBean(
        boolean pRematchExistsForProduct, List<ProductVo> pPotentialRematchProducts, boolean pDefaultToFirstProduct) {

        // See method contract. Trust but verify.
        if (pDefaultToFirstProduct && pPotentialRematchProducts.isEmpty()) {
            throw new IllegalArgumentException(
                "pDefaultToFirstProduct must not be true if pPotentialRematchProducts is empty.");
        }

        rematchExistsForProduct = pRematchExistsForProduct;
        potentialRematchProducts = pPotentialRematchProducts;
        defaultToFirstProduct = pDefaultToFirstProduct;
    }
    
    /**
     * Returns true if there already exists an unsent rematch for a given rematch's from product, false otherwise.
     * @return see description
     */
    public boolean getRematchExistsForProduct() {
        return rematchExistsForProduct;
    }

    /**
     * Returns the list of potential rematches for a given rematch's from product.
     * @return see description
     */
    public List<ProductVo> getPotentialRematchProducts() {
        return potentialRematchProducts;
    }

    /**
     * Indicates whether the first product in {@link #getPotentialRematchProducts()} should be the default selection.
     * False indicates that there should be no default selection. Will only return true if {@link
     * #getPotentialRematchProducts()} is non-empty.
     * @return see description
     */
    public boolean getDefaultToFirstProduct() {
        return defaultToFirstProduct;
    }

}
