package gov.va.med.pharmacy.peps.service.common.capability.impl;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.hibernate.ScrollableResults;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.FdbGcnseqnoPemVVo;
import gov.va.med.pharmacy.peps.common.vo.FdbGcnseqnoPlblwVVo;
import gov.va.med.pharmacy.peps.common.vo.FdbMonographPemVVo;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.NationalSettingVo;
import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.domain.common.capability.NationalSettingDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ProductDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfOutgoingDifferencesDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.domain.common.utility.FdbGcnseqnoPemVDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.FdbGcnseqnoPlblwVDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.FdbMonographPemVDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.FdbPlblwarningsVDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.MessageDataCapability;
import gov.va.med.pharmacy.peps.service.common.capability.NdfUpdateProcessCapability;
import gov.va.med.pharmacy.peps.service.common.capability.WordProcessingFieldHelper;
import gov.va.med.pharmacy.peps.service.common.update.impl.NdfRejectFileConsolidator;
import gov.va.med.pharmacy.peps.service.common.utility.MessageFormatter;
import gov.va.med.pharmacy.peps.service.common.utility.NDFUpdateFileMapping;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateMonographPemMapping;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateNdfDataMapping;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdatePlblwarningsMapping;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateVaProductAssociationsMapping;

/**
 * Primary implementation of {@link NdfUpdateProcessCapability}.
 */
public class NdfUpdateProcessCapabilityImpl implements NdfUpdateProcessCapability {

    private static final int VISTA_PARENT_FIELD_LENGTH = 3;

    private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager
        .getLogger(NdfUpdateProcessCapabilityImpl.class);
    
    private FdbMonographPemVDomainCapability fdbMonographPemVDomainCapability;
    private FdbPlblwarningsVDomainCapability fdbPlblwarningsVDomainCapability;
    private FdbGcnseqnoPemVDomainCapability fdbGcnseqnoPemVDomainCapability;
    private FdbGcnseqnoPlblwVDomainCapability fdbGcnseqnoPlblwVDomainCapability;
    private ProductDomainCapability productDomainCapability;
    private EplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao; 
    private NationalSettingDomainCapability nationalSettingDomainCapability;
    private NdfUpdateMonographPemMapping ndfUpdateMonographPemMapping;
    private NdfUpdatePlblwarningsMapping ndfUpdatePlblwarningsMapping;
    private NdfUpdateVaProductAssociationsMapping ndfUpdateVaProductAssociationsMapping;
    private NdfUpdateNdfDataMapping ndfUpdateNdfDataMapping;
    private NdfUpdateProcessMiscData ndfUpdateProcessMiscData;
    private MessageDataCapability messageDataCapability;
    private WordProcessingFieldHelper wordProcessingFieldHelper;
    private NdfRejectFileConsolidator ndfRejectFileConsolidator;
    
    private long[] languageTypes = new long[] { 1, 2 };
    private Long maxMonographId;
    private static final long ENGLISH = 1;
    private static final String PARENT = ".01";
    private static final String CHILD = ",.01";
    
    /**
     * The last set of outgoing differences retrieved. This variable is <em>not</em> a good idea.
     */
    private List<EplNdfOutgoingDifferences> returnedOutgoingDifferences = null;

    
    /**
     * Autowired constructor.
     * 
     * @param fdbMonographPemVDomainCapability fdbMonographPemVDomainCapability
     * @param fdbPlblwarningsVDomainCapability fdbPlblwarningsVDomainCapability
     * @param fdbGcnseqnoPlblwVDomainCapability fdbGcnseqnoPlblwVDomainCapability
     * @param fdbGcnseqnoPemVDomainCapability fdbGcnseqnoPemVDomainCapability
     * @param productDomainCapability productDomainCapability
     * @param eplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao
     * @param nationalSettingDomainCapability nationalSettingDomainCapability
     * @param ndfUpdateProcessMiscData ndfUpdateProcessMiscData
     * @param messageDataCapability messageDataCapability
     * @param wordProcessingFieldHelper wordProcessingFieldHelper
     * @param ndfRejectFileConsolidator ndfRejectFileConsolidator
     */
    @Autowired
    public NdfUpdateProcessCapabilityImpl(
        FdbMonographPemVDomainCapability fdbMonographPemVDomainCapability,
        FdbPlblwarningsVDomainCapability fdbPlblwarningsVDomainCapability,
        FdbGcnseqnoPlblwVDomainCapability fdbGcnseqnoPlblwVDomainCapability,
        FdbGcnseqnoPemVDomainCapability fdbGcnseqnoPemVDomainCapability,
        ProductDomainCapability productDomainCapability,
        EplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao,
        NationalSettingDomainCapability nationalSettingDomainCapability,
        NdfUpdateProcessMiscData ndfUpdateProcessMiscData,
        MessageDataCapability messageDataCapability,
        WordProcessingFieldHelper wordProcessingFieldHelper,
        NdfRejectFileConsolidator ndfRejectFileConsolidator
    ) {
        this.fdbMonographPemVDomainCapability = fdbMonographPemVDomainCapability;
        this.fdbPlblwarningsVDomainCapability = fdbPlblwarningsVDomainCapability;
        this.fdbGcnseqnoPlblwVDomainCapability = fdbGcnseqnoPlblwVDomainCapability;
        this.fdbGcnseqnoPemVDomainCapability = fdbGcnseqnoPemVDomainCapability;
        this.productDomainCapability = productDomainCapability;
        this.eplNdfOutgoingDifferencesDao = eplNdfOutgoingDifferencesDao;
        this.nationalSettingDomainCapability = nationalSettingDomainCapability;
        this.ndfUpdateProcessMiscData = ndfUpdateProcessMiscData;
        this.messageDataCapability = messageDataCapability;
        this.wordProcessingFieldHelper = wordProcessingFieldHelper;
        this.ndfRejectFileConsolidator = ndfRejectFileConsolidator;
    }
    
    @Override
    public void addDataToFile(
        NdfUpdateFileVo newFileVo, NdfUpdateProcessFile updateFile) throws IOException {

        final Calendar gc = GregorianCalendar.getInstance();
        final Date startDTM = gc.getTime();
   
        LOG.info("Writing data to file.");
        //for local development testing, it is handy to comment out the PEM, Label, and Map Data.
        processPemData(updateFile);
        processWarningLabelData(updateFile);
        processVaProductMapData(updateFile);
        processNdfData(startDTM, updateFile);
        processMiscellaneousNdfDataItems(startDTM, updateFile);
        processTextItems(updateFile);
        updateFile.closeOutputFile();  

        // Associate the changes with the update file immediately afterwards.
        LOG.info("Associating data to file.");
        updateNdfUpdateFileFk(newFileVo);
        updateRematchNdfUpdateFileFk(startDTM, newFileVo);
    }

    @Override
    public void removeAllFileAssociations(Long fileSeqNo) {
        ndfUpdateProcessMiscData.getEplRematchSuggestionDao().removeNdfUpdateFileFk(fileSeqNo);        
    }

    /**
     * Write the NDF Data to the specified file. 
     * @param startDate Date of the earliest data to include in the file.
     * @param ndfUpdateFile the NDFUpdateProcessFile in which to put the data.
     * @throws IOException Thrown if issues writing to the file.
     */    
    private void processNdfData(Date startDate, NdfUpdateProcessFile ndfUpdateFile) throws IOException {
        String vistaFileNumber;
        String vistaFieldNumber;
        
        LOG.info("---------------writing ndf data-------------------");
        setNdfUpdateNdfDataMapping(new NdfUpdateNdfDataMapping());
            returnedOutgoingDifferences = eplNdfOutgoingDifferencesDao.retrieveDifferencesForOutputFile(startDate);
        
        LOG.info("ndf row count = "+ returnedOutgoingDifferences.size() + " for startDate " + startDate);
        for (int i = 0; i < returnedOutgoingDifferences.size(); i++) {
            int count = 0;
            String txnType = " ";
            if (returnedOutgoingDifferences.get(i).getVistaFieldNumber().endsWith(PARENT) 
                && !returnedOutgoingDifferences.get(i).getNewValue().equals(" ") ) { 
                txnType = 
                returnedOutgoingDifferences.get(i).getVistaFieldNumber().length() == VISTA_PARENT_FIELD_LENGTH ? PARENT : CHILD;
            }

            switch (txnType) {
                case PARENT:
                    count = 1;

                    ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                        formatHeaderRowForDATANT(returnedOutgoingDifferences.get(i).getVistaFileNumber(), count));
                    ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                        formatIenRowForDATANT(returnedOutgoingDifferences.get(i)));
                    ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                        formatHeaderRowForDATANT(returnedOutgoingDifferences.get(i).getVistaFileNumber(), count + 1));
                    ndfUpdateFile.putNextRow(returnedOutgoingDifferences.get(i).getNewValue());
                    break;
                case CHILD:
                    count = 1;
                    
                    vistaFileNumber = returnedOutgoingDifferences.get(i).getVistaFileNumber();
                    vistaFieldNumber = returnedOutgoingDifferences.get(i).getVistaFieldNumber();
                    
                    if ( wordProcessingFieldHelper.isWordProcessingField(vistaFileNumber,  vistaFieldNumber))
                    {
                        wordProcessingFieldHelper.formatWordProcessingField(
                            ndfUpdateFile, returnedOutgoingDifferences.get(i), vistaFileNumber, vistaFieldNumber);
                    } else {
                      ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                          formatHeaderRowForDATAN(returnedOutgoingDifferences.get(i).getVistaFileNumber(), count));
                      ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                          formatIenRowForDATAN(returnedOutgoingDifferences.get(i)));
                      ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                          formatHeaderRowForDATAN(returnedOutgoingDifferences.get(i).getVistaFileNumber(), count + 1));
                      ndfUpdateFile.putNextRow(returnedOutgoingDifferences.get(i).getNewValue());
                    }
                    break;

                default:
                    count = 1;

                    vistaFileNumber = returnedOutgoingDifferences.get(i).getVistaFileNumber();
                    vistaFieldNumber = returnedOutgoingDifferences.get(i).getVistaFieldNumber();
                    
                    if ( wordProcessingFieldHelper.isWordProcessingField(vistaFileNumber,  vistaFieldNumber))
                    {
                        wordProcessingFieldHelper.formatWordProcessingField(
                            ndfUpdateFile, returnedOutgoingDifferences.get(i), vistaFileNumber, vistaFieldNumber);
                    } else {
                      ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                          formatHeaderRowForDATAO(returnedOutgoingDifferences.get(i).getVistaFileNumber(), count));
                      ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                          formatIenRowForDATAO(returnedOutgoingDifferences.get(i)));
                      ndfUpdateFile.putNextRow(ndfUpdateNdfDataMapping.
                          formatHeaderRowForDATAO(returnedOutgoingDifferences.get(i).getVistaFileNumber(), count + 1));
                      if(returnedOutgoingDifferences.get(i).getNewValue().equals(" "))
                      {
                          ndfUpdateFile.putNextRow("");
                      }else{
                          ndfUpdateFile.putNextRow(returnedOutgoingDifferences.get(i).getNewValue());
                      }
                    }
                    break;
            }
        }
    }
    
    /**
     * Process 5000 series data and data reports.
     * 
     * @param startDTM the start date of the update file process
     * @param ndfUpdateFile the ndf update file
     * @throws IOException Thrown if issues writing to the file.
     */
    private void processMiscellaneousNdfDataItems(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException {
        
        LOG.info("---------------writing misc ndf data-------------------");
       
        //5000.68 data inactive product list
        ndfUpdateProcessMiscData.inactiveProducts(startDTM, ndfUpdateFile);
        
        //5000.3 data inactive generic name list
        ndfUpdateProcessMiscData.inactiveGenericName(startDTM, ndfUpdateFile);
        
        //5000.4 Strength Changes report
        ndfUpdateProcessMiscData.productStrengthChanges(startDTM, ndfUpdateFile);
        
        //5000.5 National Formulary changes report
        ndfUpdateProcessMiscData.productFormularyChanges(startDTM, ndfUpdateFile);
        
        //5000.6 Drug Class changes report
        ndfUpdateProcessMiscData.productDrugClassChanges(startDTM, ndfUpdateFile);
        
        //REM
        ndfUpdateProcessMiscData.rematchProducts(startDTM, ndfUpdateFile);
        
        //5000.7 Unmark for CMOP changes report
        ndfUpdateProcessMiscData.productUnmarkForCmopChanges(startDTM, ndfUpdateFile); 
        
        //message data report
        writeDataUpdateForNdfReport(startDTM, ndfUpdateFile);
        
        //message2 data report
        writeUpdatedInteractionsReport(startDTM, ndfUpdateFile);
    }
    
    private void writeDataUpdateForNdfReport(final Date startDTM, final NdfUpdateProcessFile ndfUpdateFile)
        throws IOException {
        final List<String> reportData = messageDataCapability.buildDataUpdateForNdfReport(startDTM);
        MessageFormatter formatter = new MessageFormatter(new Date(), reportData);
        ndfUpdateFile.putNextRow(formatter.getFormattedMessage(NDFUpdateFileMapping.MESSAGE));
    }

    private void writeUpdatedInteractionsReport(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException {
        final List<String> result = messageDataCapability.buildUpdatedInteractionsReport(startDTM);
        final MessageFormatter formatter = new MessageFormatter(new Date(), result);
        ndfUpdateFile.putNextRow(formatter.getFormattedMessage(NDFUpdateFileMapping.MESSAGE2));
    }

    /**
     * Process VA Product Data.
     * 
     * @param ndfUpdateFile the ndf update file
     * @throws IOException Thrown if issues writing to the file.

     */
    private void processVaProductMapData(NdfUpdateProcessFile ndfUpdateFile) throws IOException {
        LOG.info("---------------writing va product data-------------------");

        setNdfUpdateVaProductAssociationsMapping(new NdfUpdateVaProductAssociationsMapping());
        List<Object[]> returnedProducts = productDomainCapability.retrieveGcnNdcMap();
        LOG.info(" product count = " + returnedProducts.size());
        StringBuffer writerBuffer = new StringBuffer(200);
        StringBuffer dataBuffer = new StringBuffer(200);
        int lineCount = 1;

        for (Object[] product : returnedProducts) {
            BigDecimal productIen = (BigDecimal) product[0];
            BigDecimal gcn = (BigDecimal) product[1];
            String ndc = (String) product[2];
            String productNdc = (ndc == null ? "": ndc);

            dataBuffer.append(ndfUpdateVaProductAssociationsMapping.formatTextDataRow(
                // Using the current GCNSEQNO as the previous one as well. This is technically incorrect but the
                // "PREVIOUS GCNSEQNO" field is no longer used in VistA.
                productIen.longValue(), gcn.longValue(), gcn.longValue(), productNdc));
            if ((writerBuffer.length() + dataBuffer.length()) < 180) {
                writerBuffer.append(dataBuffer);
                dataBuffer.setLength(0);
            } else {
                writerBuffer.append(dataBuffer);
                dataBuffer.setLength(0);
                ndfUpdateFile.putNextRow(ndfUpdateVaProductAssociationsMapping.formatTextTempRow(lineCount));
                ndfUpdateFile.putNextRow(writerBuffer.toString());
                lineCount++;
                writerBuffer.setLength(0);
                dataBuffer.setLength(0);
            }
        }
        if(writerBuffer.length() != 0){
            ndfUpdateFile.putNextRow(ndfUpdateVaProductAssociationsMapping.formatTextTempRow(lineCount));
            ndfUpdateFile.putNextRow(writerBuffer.toString());
        }
    }
    /**
     * Process PMI Data.
     * 
     * @param ndfUpdateFile the ndf update file
     * @throws IOException Thrown if issues writing to the file.
     */
    private void processPemData(NdfUpdateProcessFile ndfUpdateFile) throws IOException {

        LOG.info("---------------writing PMI data-------------------");
        setNdfUpdateMonographPemMapping(new NdfUpdateMonographPemMapping());

        //process English and Spanish monographs 50.621 50.622
        for (long language : languageTypes) {

            ndfUpdateMonographPemMapping.setPemOutputLanguageType(language);
            setMaxMonographId(fdbMonographPemVDomainCapability.retrieveMaxMonographidByLanguageid(language));
            Long monographCount = fdbMonographPemVDomainCapability.retrieveMonographCountByLanguageid(language);

            ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.
                formatTempRowForHeader());

            ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.
                formatDataRowForPmiHeader(getMaxMonographId(), monographCount));

            if (language == ENGLISH) {
                ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.
                    formatPmiRowForPmiTimeStamp());

                ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatDataRowForPmiDateStamp());
            }

            processMonographLines(language, ndfUpdateFile);

            //title monograph map 
            List<FdbMonographPemVVo> returnedTitles =
                fdbMonographPemVDomainCapability.retreiveMonographTitlesBylanguageid(language);
            for (int i = 0; i < returnedTitles.size(); i++) {
                ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatRowForMonographTitleGcn(returnedTitles.get(i)));
                ndfUpdateFile.putNextRow(" ");
            }
            returnedTitles = null;
        }

        //-----Begin MAP output-----
        //gcn,monographid map English 50.623 and Spanish 50.624
        LOG.info("-------monograph gcn map-----");
        for (long language : languageTypes) {

            ndfUpdateMonographPemMapping.setPemOutputLanguageType(language);
            List<FdbGcnseqnoPemVVo> returnedGcnMap = fdbGcnseqnoPemVDomainCapability.retreiveGcnsMonographids(language);
            setMaxMonographId(fdbMonographPemVDomainCapability.retrieveMaxMonographidByLanguageid(language));

            ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatPmiRowForMapHeader());
            ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatDataRowForMapHeader(getMaxMonographId(),
                returnedGcnMap.size()));

            int mapRowCount = 1;
            for (int i = 0; i < returnedGcnMap.size(); i++) {
                ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatPmiRowForGcnMonographMap(mapRowCount));
                ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatDataRowForGcnMonographMap(
                    returnedGcnMap.get(i).getGcnSeqNo(), returnedGcnMap.get(i).getMonographId()));
                mapRowCount++;
            }

            Long maxGcn = fdbGcnseqnoPemVDomainCapability.retrieveMaxGcn(language);
            Long totalGcns = fdbGcnseqnoPemVDomainCapability.retrieveGcnCount(language);

            //gcn count map English 50.623           
            LOG.info("-------gcn count-----");
            ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatHeaderMonographIdCountRow(maxGcn,
                Long.valueOf(totalGcns)));
            ndfUpdateFile.putNextRow(" ");
            for (int i = 0; i < returnedGcnMap.size(); i++) {
                int gcnCount = i + 1;
                ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatMonographIdCountRow(returnedGcnMap.get(i)
                    .getGcnSeqNo(), gcnCount));
                ndfUpdateFile.putNextRow(" ");
            }
            returnedGcnMap = null;
        }
    }

    /**
     * Process warning label data.
     * 
     * @param ndfUpdateFile the ndf update file
     * @throws IOException Thrown if issues writing to the file.
     */
    private void processWarningLabelData(NdfUpdateProcessFile ndfUpdateFile) throws IOException {

        LOG.info("---------------writing warning label data-------------------");
        setNdfUpdatePlblwarningsMapping(new NdfUpdatePlblwarningsMapping());

        //process English warning labels 50.625 and 50.626
        for (long language : languageTypes) {

            ndfUpdatePlblwarningsMapping.setPlblWOutputLanguageType(language);

            Long warningLabelCount = fdbPlblwarningsVDomainCapability.retrieveWarningLabelCount(language);
            String maxLwid = fdbPlblwarningsVDomainCapability.retrieveMaxLwid(language);
            ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatTempRowForHeader());
            ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatDataRowForHeader(Integer.valueOf(maxLwid), 
                warningLabelCount));

            writeWarningLabelById(language, ndfUpdateFile);

            //plblw id count map  50.625 and 50.626
            List<String> plblwIdList = fdbGcnseqnoPlblwVDomainCapability.retreiveLwIdMapByLanguageId(language);
            for (int i = 0; i < plblwIdList.size(); i++) {
                int plblListCount = i + 1;
                ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatLwidMapRow(plblwIdList.get(i), plblListCount));
                ndfUpdateFile.putNextRow(" ");
            }
            plblwIdList = null;
        }

        //---Begin Map data--------
        //map gcn, lwid, priority 50.627
        List<FdbGcnseqnoPlblwVVo> gcnLwidPriorityList = fdbGcnseqnoPlblwVDomainCapability.retreiveGcnLwIdAndPriorityMap();
        int gcnMapCount = gcnLwidPriorityList.size();
        ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatPmiRowForMap());
        ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatDataRowForMapHeader(gcnMapCount, gcnMapCount));
        int lwidMapCount = 1;
        for (int i = 0; i < gcnLwidPriorityList.size(); i++) {
            ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatPmiRowForMapData(lwidMapCount));
            ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatDataRowForMapData(
                gcnLwidPriorityList.get(i).getGcnSeqNo(), gcnLwidPriorityList.get(i).getLwId(), gcnLwidPriorityList.get(i)
                    .getPriority()));
            lwidMapCount++;
        }
        gcnLwidPriorityList = null;

        // plblwid count map for all 50.627
        List<Long> plblwIdList = fdbGcnseqnoPlblwVDomainCapability.retreiveAllGcnsMap();
        for (int i = 0; i < plblwIdList.size(); i++) {
            int gcnCount = i + 1;
            ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatAllGcnsMapRow(plblwIdList.get(i), gcnCount));
            ndfUpdateFile.putNextRow(" ");
        }
        plblwIdList = null;
    }

    /**
     * Loads monographs from the database and writes them to the given file.
     * @param languageId the language of monographs to write
     * @param ndfUpdateFile the file to write to
     * @throws IOException if there is an I/O error while writing the data
     */
    private void processMonographLines(Long languageId, NdfUpdateProcessFile ndfUpdateFile) throws IOException {

        // Does this return "Monographids" or Monographs?
        ScrollableResults returnedMonographs = fdbMonographPemVDomainCapability.retreiveMonographidsBylanguageid(languageId);
        
        String previousSectionCode = "";
        int sectionCount = 1;
        while (returnedMonographs.next()) {
            BigDecimal monographId = (BigDecimal) returnedMonographs.get(0);
            String sectionCode = (String) returnedMonographs.get(1);
            String lineOfText = (String) returnedMonographs.get(2);
            BigDecimal lineOfTextCount = (BigDecimal) returnedMonographs.get(3);

            if (!"A".equals(sectionCode)) {
                if (!previousSectionCode.equals(sectionCode))
                {
                    sectionCount = 1;
                    if (NdfUpdateMonographPemMapping.TITLE_INDICATOR.equals(sectionCode)) {
                        ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatPmiRowForTitleDateStamp(
                            monographId.longValue()));
                    } else {
                        ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.
                            formatPmiRowForTextDateStamp(monographId.longValue(), sectionCode));
                        ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.
                            formatDataRowForTextDateStamp(lineOfTextCount.longValue()));
                    }
                }
                writeMonographLines(monographId.longValue(), sectionCode, lineOfText, sectionCount, ndfUpdateFile);
                if (NdfUpdateMonographPemMapping.TITLE_INDICATOR.equals(sectionCode)) {
                    ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatPmiRowForTextDateStamp(
                        monographId.longValue(), NdfUpdateMonographPemMapping.BLANK_INDICATOR));

                    ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatDataRowForTextDateStamp(1));

                    ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatPmiRowForText(
                        monographId.longValue(), NdfUpdateMonographPemMapping.BLANK_INDICATOR, 1));

                    ndfUpdateFile.putNextRow(" ");
                }
                sectionCount++;
                previousSectionCode = sectionCode;
            }
        }
        returnedMonographs = null;
    }
    
    /**
     * Process default text data.
     *
     * @param ndfUpdateFile the ndf update file
     * @throws IOException Thrown if issues writing to the file.
     * @throws Exception Thrown if issues other than writing to the file.
     */
    private void processTextItems(NdfUpdateProcessFile ndfUpdateFile) throws IOException {
        
        NationalSettingVo returnedNationalSetting = 
            nationalSettingDomainCapability.retrieve(NationalSetting.NDF_OUTGOING_FILE_TEXT.toString());
       
        String defaultText = 
            returnedNationalSetting.getStringValue() != null?returnedNationalSetting.getStringValue():"missing text";
         
        String[] splits = defaultText.split("\\|");
        int lineCount = 1;
        for(String text: splits){
            ndfUpdateFile.putNextRow("TEXT^"+ lineCount);
            ndfUpdateFile.putNextRow(text);
            lineCount++;

        }
        ndfUpdateFile.putNextRow("TEXT^"+ lineCount);
        ndfUpdateFile.putNextRow(" ");
        ndfUpdateFile.putNextRow("TREC^1");
        ndfUpdateFile.putNextRow("2");
    }

    /**
     * Writes monograph section. Not all sections are available for all
     * language translations.
     * 
     * @param ndfUpdateMonographPemList
     *            list of NdfUpdateMonographPemVo's
     * @param ndfUpdateFile
     *           ndfUpdateFile
     * @throws IOException Thrown if issues writing to the file.
     */
    private void writeMonographLines(Long monographId, String sectionCode, String lineOfText, int sectionCount,  
        NdfUpdateProcessFile ndfUpdateFile) throws IOException   {

        if (!NdfUpdateMonographPemMapping.TITLE_INDICATOR.equals(sectionCode))
        {
            ndfUpdateFile.putNextRow(ndfUpdateMonographPemMapping.formatPmiRowForText(monographId, sectionCode,sectionCount));
        }
        ndfUpdateFile.putNextRow(lineOfText.trim());
    }

    /**
     * Writes warning label text.
     * 
     * @param ndfUpdatePlblList
     *             List of FdbPlblwarningsVVo's
     * @param ndfUpdateFile
     *           ndfUpdateFile
     * @throws IOException Thrown if issues writing to the file.
     */
    private void writeWarningLabelById(Long language, NdfUpdateProcessFile ndfUpdateFile) throws IOException {

        String previousId = "";
        int lineCount = 0;

        List<Object[]> ndfUpdatePlblList = fdbPlblwarningsVDomainCapability.retreiveWarningLabelByLanguageId(language);
       
        int count = 0;
        for (int i = 0; i < ndfUpdatePlblList.size(); i++)
        {
            String lwid = (String) ndfUpdatePlblList.get(i)[0];
            String warningText = (String) ndfUpdatePlblList.get(i)[1];
            BigDecimal totalTextLines = (BigDecimal) ndfUpdatePlblList.get(i)[2];
            if (!previousId.equals(lwid)) {
                lineCount = 0;
                count = count + 1;
                ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatHeaderRowForLwid(count));
                ndfUpdateFile.putNextRow(lwid);
                ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatHeaderRowForDateStamp(lwid));
                ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatDataRowForDateStamp(totalTextLines.intValue()));
            }
            lineCount++;
            previousId = lwid;
            ndfUpdateFile.putNextRow(ndfUpdatePlblwarningsMapping.formatHeaderRowForText(lwid, lineCount));
            ndfUpdateFile.putNextRow(warningText.trim());
        }
        ndfUpdatePlblList = null;
    }
    
    @Override
    public void processRejectedFile(NdfUpdateFileVo rejectedNdfUpdateFileVo) {
        //Get pending outgoing differences.
        List<EplNdfOutgoingDifferences> pendingDiffs = 
            eplNdfOutgoingDifferencesDao.findPendingRecords();
        
        Long fileId = Long.valueOf(rejectedNdfUpdateFileVo.getFileId());
        
        //Get rejected outgoing differences.
        List<EplNdfOutgoingDifferences> rejectedDiffs = 
            eplNdfOutgoingDifferencesDao.findByNdfUpdateFileId(fileId);
        
        //Delete all pending and rejected.
        eplNdfOutgoingDifferencesDao.deletePending();
        eplNdfOutgoingDifferencesDao.deleteByNdfUpdateFileId(fileId);
        removeAllFileAssociations(fileId);
        
        //Reconcile pending and rejected.
        List<EplNdfOutgoingDifferences> mergedDiffs = 
            ndfRejectFileConsolidator.reconcile(pendingDiffs, rejectedDiffs);
        
        //Detach objects from session so Hibernate will treat them as 
        //new entities.
        mergedDiffs = eplNdfOutgoingDifferencesDao.evictAll(mergedDiffs);
        
        //Sort into a logical insertion order.
        Collections.sort(mergedDiffs, (diff1, diff2) -> {
            String leftValue = diff1.getVistaFileNumber() 
                + diff1.getVistaIen() 
                + diff1.getVistaFieldNumber();
            String rightValue = diff2.getVistaFileNumber() 
                + diff2.getVistaIen() 
                + diff2.getVistaFieldNumber();
            return leftValue.compareTo(rightValue);
        });
        
        //Write new pending outgoing differences.
        for(EplNdfOutgoingDifferences diff : mergedDiffs) {
            eplNdfOutgoingDifferencesDao.save(diff);
        }
    }
    
    /**
     * Update the ndf_update_file_fk field with seq number for this processing cycle.
     *
     * @param ndfUpdateFileVo the ndf update file
     */
    private void updateNdfUpdateFileFk(NdfUpdateFileVo ndfUpdateFileVo){
        eplNdfOutgoingDifferencesDao.updateNdfUpdateFileFk(returnedOutgoingDifferences, ndfUpdateFileVo);
    }
    
    /**
     * Updates the File reference on the Rematch data. 
     * @param startDate Date of the earliest data to include in the file
     * @param ndfUpdateFileVo the ndf update file to associate to the rematch data.
     */
    private void updateRematchNdfUpdateFileFk(Date startDate, NdfUpdateFileVo ndfUpdateFileVo){          
        ndfUpdateProcessMiscData.getEplRematchSuggestionDao().updateRematchNdfUpdateFileFk(startDate, ndfUpdateFileVo);
    }

    public NdfUpdateMonographPemMapping getNdfUpdateMonographPemMapping() {
        return ndfUpdateMonographPemMapping;
    }

    public void setNdfUpdateMonographPemMapping(NdfUpdateMonographPemMapping ndfUpdateMonographPemMapping) {
        this.ndfUpdateMonographPemMapping = ndfUpdateMonographPemMapping;
    }

    public NdfUpdatePlblwarningsMapping getNdfUpdatePlblwarningsMapping() {
        return ndfUpdatePlblwarningsMapping;
    }

    public void setNdfUpdatePlblwarningsMapping(NdfUpdatePlblwarningsMapping ndfUpdatePlblwarningsMapping) {
        this.ndfUpdatePlblwarningsMapping = ndfUpdatePlblwarningsMapping;
    }

    public NdfUpdateVaProductAssociationsMapping getNdfUpdateVaProductAssociationsMapping() {
        return ndfUpdateVaProductAssociationsMapping;
    }

    public void setNdfUpdateVaProductAssociationsMapping(
        NdfUpdateVaProductAssociationsMapping ndfUpdateVaProductAssociationsMapping) {
        this.ndfUpdateVaProductAssociationsMapping = ndfUpdateVaProductAssociationsMapping;
    }

    /**
     * Get the ndfUpdateNdfDataMapping.
     * 
     * @return the ndfUpdateNdfDataMapping
     */
    public NdfUpdateNdfDataMapping getNdfUpdateNdfDataMapping() {
        return ndfUpdateNdfDataMapping;
    }

    /**
     * Set the ndfUpdateNdfDataMapping.
     * 
     * @param ndfUpdateNdfDataMapping the ndfUpdateNdfDataMapping to set
     */
    public void setNdfUpdateNdfDataMapping(NdfUpdateNdfDataMapping ndfUpdateNdfDataMapping) {
        this.ndfUpdateNdfDataMapping = ndfUpdateNdfDataMapping;
    }

    /**
     * Get the maxEnglishMonographId.
     * 
     * @return the maxEnglishMonographId
     */
    public Long getMaxMonographId() {
        return maxMonographId;
    }

    /**
     * Set the maxMonographId.
     * 
     * @param maxMonographId the maxMonographId to set
     */
    public void setMaxMonographId(Long maxMonographId) {
        this.maxMonographId = maxMonographId;
    }
    
    @Override
    public List<String> getPreviewUpdateFileReportData(Date startDTM) {
        
        LOG.info("---------------generating update file preview report-------------------");
                 
        return messageDataCapability.generateUpdateFilePreviewReport(startDTM);
    }
}
