/**
 * Source file created in 2007 by Southwest Research Institute
 */

package gov.va.med.pharmacy.peps.service.common.migration;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

/**
 * A POJO to represent the state, progress count, and the total count of the migration.
 */
public class MigrationStatus {

  /** Migration status if no migration has yet been run. */
  public static final String INITIAL = "INITIAL";

  /** Migration status during processing. */
  public static final String INPROGRESS = "INPROGRESS";

  /** Migration status when it's finished. */
  public static final String DONE = "DONE";

  private int progressCount;

  private int totalCount;
  /**
   * Status of migration.
   */
  private String status;

  /**
   * Constructs an instance with default values for all properties.
   */
  public MigrationStatus() {
    progressCount = 0;
    totalCount = 0;
    status = INITIAL;
  }

  /**
   * get the status.
   * 
   * @return String
   */
  public final String getStatus() {
    return status;
  }

  /**
   * set the status.
   * 
   * @param status String
   */
  public final void setStatus(String status) {
    this.status = status;
  }

  /**
   * Get the number of records that have been processed so far.
   *
   * @return progress count
   */
  public final int getProgressCount() {
    return progressCount;
  }

  /**
   * Set the number of records that have been processed so far.
   * 
   * @param progressCount the value to set
   */
  public final void setProgressCount(final int progressCount) {
    this.progressCount = progressCount;
  }

  /**
   * Get the total number of records in the migration.
   *
   * @return total count
   */
  public final int getTotalCount() {
    return totalCount;
  }

  /**
   * Set the total number of records in the migration.
   * 
   * @param totalCount the value to set
   */
  public final void setTotalCount(final int totalCount) {
    this.totalCount = totalCount;
  }
  
  @Override
  public String toString() {
    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
        .append("status", getStatus())
        .append("progressCount", getProgressCount())
        .append("totalCount", getTotalCount())
        .toString();
  }
}
