/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.scheduler.jobs;


import java.util.Date;

import org.apache.logging.log4j.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;

import gov.va.med.pharmacy.peps.service.common.scheduler.tasks.FdbAddTaskImpl;


/**
 * 
 * Quartz Job to process new FDB Packaged Drugs added by FDB since the last successful execution of this job.
 *
 */
@DisallowConcurrentExecution
public class FdbAddJob extends QuartzJobBean implements Job {
    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(FdbAddJob.class);
    
    private FdbAddTaskImpl fdbAddTask;
    
    /**
     * Contructor for FdbAddJob.
     *
     */
    public FdbAddJob() {
    }

    /**
     * Main execution method for FdbAddJob.
     * @param context - JobExecutionContext
     * @throws JobExecutionException JobExecutionException
     */
    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        
        LOG.debug("\n");
        LOG.info("-------------------------- Executing FDBAdd Job ------------------------------------");
        LOG.debug(" Time:  " +  new Date() + "                                                        ");
        LOG.debug(" Next fire time: " + context.getNextFireTime() + "                                 ");
        
        try{
          
          fdbAddTask.runFdbAddProcess();
        
        } finally{
          LOG.info("-------------------------- End of FDBAdd Job ----------------------------------------");
        }
    }
    
    /**
     * gets FdbAddTask.
     * @return the fdbAddTask
     */
    public FdbAddTaskImpl getFdbAddTask() {
        return fdbAddTask;
    }

    /**
     * sets FdbAddTask.
     * @param fdbAddTask the fdbAddTask to set
     */
    public void setFdbAddTask(FdbAddTaskImpl fdbAddTask) {
        this.fdbAddTask = fdbAddTask;
    }
    
}
