package gov.va.med.pharmacy.peps.service.common.session;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.PaginatedList;
import gov.va.med.pharmacy.peps.common.vo.StatusVo;
import gov.va.med.pharmacy.peps.common.vo.UpdateSearchCriteria;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

import java.util.List;

/**
 * High-level business operations on NDF Update Files.
 */
public interface NDFUpdateFileService {
    
    /**
     * 
     * 
     * @return List<NdfUpdateFileVo> all UpdateFiles
     */
    List<NdfUpdateFileVo> retrieveFileUpdates();
    
    /**
     * 
     * 
     * @return List<NdfUpdateFileVo> all UpdateFiles
     */
    PaginatedList<NdfUpdateFileVo> retrievePagedFileUpdates(UpdateSearchCriteria searchCriteria);
    
    /**
     * 
     * 
     * @return List<NdfUpdateFileVo> pending UpdateFiles
     */
    List<NdfUpdateFileVo> retrievePendingNdfUpdate();
    
    /**
     * 
     * 
     * @return NdfUpdateFileVo pending UpdateFile
     */
    NdfUpdateFileVo retrieveById(Long id);
    
    /**
     * 
     * 
     * @return List<NdfUpdateFileVo> pending UpdateFiles
     */
    List<NdfUpdateFileVo> retrieveInitiatedNdfUpdate();

    /**
     *
     * @return List<StatusVo> all File Update Statuses 
     */
    List<StatusVo> retrieveAllFileStatuses();

    /**
     * Creates and sends a new on-demand update file asynchronously.
     * 
     * @param user the user generating the file
     * @return the created {@link NdfUpdateFileVo}. Will have status {@link StatusVo#INITIATED}.
     * @throws ValidationException if there is already a file going through the approval process
     */
    NdfUpdateFileVo createOnDemandFileAsync(UserVo user) throws ValidationException;

    /**
     * Approve.  Updates the status and transmits the file to the production folder.
     *
     * @param toUpdate the to update
     * @param user the user
     * @throws ValidationException if the file transmission to the production folder failed
     */
    void approve(NdfUpdateFileVo toUpdate, UserVo user) 
            throws ValidationException;
    
    /**
     * Reject.  Updates the status, moves the file to the reject folder, and remove the associations between the File 
     * and the data.
     *
     * @param toUpdate the to update
     * @param user the user
     * @throws ValueObjectValidationException the value object validation exception
     * @throws ValidationException the validation exception
     */
    void reject(NdfUpdateFileVo toUpdate, UserVo user)
            throws ValueObjectValidationException, ValidationException;   
}
