package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.io.File;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.MigrationRow;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;

/**
 * The Interface for a Migration Service.
 *
 * @param <R> the generic type of MigrationRow processed
 */
public interface MigrationService<R extends MigrationRow> {

    /**
     * Process.
     *
     * @param rows the rows
     * @return the list
     */
    List<R> process(List<R> rows);

    /**
     * Initialize.
     *
     * @param rowCount the row count
     * @param migrationUser the migration user
     * @throws MigrationInProgressException the migration in progress exception
     */
    void initialize(int rowCount, UserVo migrationUser) throws MigrationInProgressException;

    /**
     * Gets the status.
     *
     * @return the status
     */
    MigrationStatus getStatus();

    /**
     * Gets the result file.
     *
     * @return the result file
     */
    File getResultFile();

    /**
     * Write result file.
     *
     * @param rows the rows
     * @param sheetName the sheet name to apply
     */
    void writeResultFile(List<R> rows, String sheetName);

}
