package gov.va.med.pharmacy.peps.service.common.utility;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;

/**
 * Transforms a list of strings representing a message data block into the format needed for the
 * update file.
 */
public class MessageFormatter {

    private static final String NEWLINE = System.getProperty("line.separator");

    private Date messageDate = null;
    private List<String> plainMessages = new ArrayList<String>();

    /**
     * Constructs an instance.
     * 
     * @param d the message generation date. This is added to the report header in the update file.
     * @param messageData the message data to format.
     */
    public MessageFormatter(Date d, List<String> messageData) {
        messageDate = d;
        plainMessages.addAll(messageData);
    }
    
    /**
     * Transforms the message block provided in the constructor into the format needed for the
     * update file.
     * @param tag the "section tag" of the message block (e.g., "MESSAGE" or "MESSAGE2")
     * @return the content to add to the update file as a String
     */
    public String getFormattedMessage(final String tag) {
        return apply(tag).stream().collect(Collectors.joining(NEWLINE));
    }

    /**
     * Transitional step: adds the necessary formatting but returns results as a list of lines.
     */
    private List<String> apply(String tag) {
        List<String> result = new ArrayList<String>();

        // add overall message header..
        result.addAll(getMessageHeader(tag));

        for (int i=0; i < plainMessages.size(); i++) {
            result.add(tag + (i+1) + NDFUpdateFileMapping.ENDER);
            result.add(plainMessages.get(i));
        }
        return result;
    }

    private String getMumpsFormattedDateString() {
        return MumpsConverter.convertDate(messageDate);
    }

    private List<String> getMessageHeader(String tag) {
        List<String> result = new ArrayList<String>();
        result.add(tag + "0"
                + NDFUpdateFileMapping.ENDER);
        result.add(NDFUpdateFileMapping.DOUBLE_ROW_FIELD_SEPARATOR
                + plainMessages.size()
                + NDFUpdateFileMapping.ROW_FIELD_SEPARATOR
                + plainMessages.size()
                + NDFUpdateFileMapping.ROW_FIELD_SEPARATOR 
                + getMumpsFormattedDateString());
        return result;
    }

}
