package gov.va.med.pharmacy.peps.service.common.utility;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import gov.va.med.pharmacy.peps.common.exception.CsvFileException;
import gov.va.med.pharmacy.peps.common.vo.DateFormat;
import gov.va.med.pharmacy.peps.common.vo.ReportFdbCopayChangeVo;
import gov.va.med.pharmacy.peps.common.vo.ReportType;

/**
 * This class creates the CSV downloadable text file with the
 * FDB Copay Change report data.
 */
public class ReportFdbCopayChangeCsvFile extends CsvFile {
    
    /**
     * The directory where the CSV file will be stored on the server.
     */
    private static final String PATH = "./tmp/";
    
    /**
     * The name of the CSV file.
     */
    private static final String FILENAME = ReportType.FDB_COPAY_CHANGE.getView() + ".csv";
    
    /**
     * The names of the fields included in the report, in the order
     * they will be written in the header row of the output.
     */
    private static final String[] FIELD_NAMES = {
        "VA_PRODUCT_NAME",
        "VA_ORDERABLE_ITEM_NAME",
        "GCNSEQNO",
        "INACTIVE_DATE",
        "FDB_TIER_LEVEL",
        "FUTURE_TIER_LEVEL",
        "FEDERAL_LEGEND_CODE"
    };
    
    /**
     * Creates file and writes data.
     * 
     * @param dataList The report data to be written to the output file.
     * @throws MigrationException if there is a problem writing the output file.
     */
    public final void createFile(List<ReportFdbCopayChangeVo> dataList) throws CsvFileException {
        openAndWriteHeader(PATH + FILENAME, FIELD_NAMES);
        
        SimpleDateFormat formatter = new SimpleDateFormat(DateFormat.DAY_MONTH_YEAR.getFormat());
        
        for(ReportFdbCopayChangeVo vo : dataList) {
            List<String> row = new ArrayList<String>();
            
            row.add(vo.getVaProductName() == null ? "" : vo.getVaProductName());
            row.add(vo.getOiName() == null ? "" : vo.getOiName());
            row.add(Objects.toString(vo.getGcnSequenceNumber(), ""));
            row.add(vo.getInactivationDate() == null ? "" : formatter.format(vo.getInactivationDate()));
            row.add(vo.getFdbTierLevel() == null ? "" : vo.getFdbTierLevel());
            row.add(vo.getFutureTierLevel() == null ? "" : vo.getFutureTierLevel());
            row.add(vo.getFederalLegendCode() == null ? "" : vo.getFederalLegendCode());
            
            putNextRow(row.toArray(new String[row.size()]));
        }
        
        closeExport();
    }

}
