package gov.va.med.pharmacy.peps.service.national.updatefile;

import gov.va.med.pharmacy.peps.common.vo.ProductReference;

/**
 * Utility class containing methods and constants for formatting the update file reports.
 */
public final class UpdateFileReportFormatter {

    /**
     * Just an empty string. This constant is useful for making code more readable, e.g.
     * <code>report.add(EMPTYLINE)</code>.
     */
    public static final String EMPTYLINE = "";

    private static final String DOUBLEDSPACE = "    ";

    /**
     * 5 spaces. Intended to separate two items on the same line.
     */
    public static final String SPACER = "     ";  // 5 spaces
    
    /**
     * Formats the given {@link ProductReference} for the update file reports.
     * @param productRef the product reference to format
     * @return the formatted String
     */
    public static String formatProductReference(final ProductReference productRef) {
        return formatPairWithSquareBraces(productRef.getVaProductName(), productRef.getCmopId());
    }
    
    /**
     * Formats the first item followed by the second item in square braces, then indents the string.
     * @param item1 the first item
     * @param item2 the second item
     * @return the formatted string
     */
    public static final String formatPairWithSquareBraces(String item1, String item2) {
        
        StringBuffer buf = new StringBuffer();
        
        buf.append(DOUBLEDSPACE);
        buf.append(item1);
        buf.append(SPACER);
        buf.append(inSquareBraces(item2));
        
      return buf.toString();
    }

    private static String inSquareBraces(String item) {
        return "[" + item + "]";
    }
    
    /**
     * Double-indents the given string. This is the standard indent level for individual products under a section
     * header.
     * 
     * @param data the string to indent
     * @return the indented string
     */
    public static String doubleIndent(String data) {
        return DOUBLEDSPACE + data;
    }

    /**
     * Indents the given string twice. Equivalent to two calls to {@link #doubleIndent(String)}.
     *  
     * @param data the string to indent
     * @return the indented string
     */
    public static String indentLevel2(String data) {
        return doubleIndent(doubleIndent(data));
    }    
    
    /**
     * No construction.
     */
    private UpdateFileReportFormatter() {
    }
}
