package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.mockito.Mockito.*;

import java.nio.channels.SeekableByteChannel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.utility.VistaFileNumber;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfOutgoingDifferencesDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplRematchDao;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;
import gov.va.med.pharmacy.peps.service.national.updatefile.ReportMessageCode;
import gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportGenerator;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

/**
 * <p>
 * Older unit tests for {@link MessageDataCapabilityImpl}. Most of these tests are disabled and they all use Mockito instead of
 * EasyMock, which is the PPS-N standard.
 * </p>
 * 
 * <p>
 * Useful tests in this class should be moved to {@link MessageDataCapabilityImplTest} and the rest deleted along with this
 * class.
 * </p>
 */
@RunWith(MockitoJUnitRunner.class)
public class MessageDataCapabilityImplOldTest {

    public static final Long LONG_20150911 = 1441975560000L;
    public static final Date DATE_20150911 = new Date(LONG_20150911);
    private static final String VISTA_FIELD_NUMBER_DOT_01 = ".01";
    private static final String BAR_DELIMITED_MSG_PT1 = "this is";
    private static final String BAR_DELIMITED_MSG_PT2 = "a test";
    private static final String BAR_DELIMITED_MSG_PT3 = "of the";
    private static final String BAR_DELIMITED_MSG_PT4 = "whatever";
    private static final List<String> SAMPLE_PADDED_HEADER_MESSAGE = Arrays.asList(
        "", BAR_DELIMITED_MSG_PT1, BAR_DELIMITED_MSG_PT2, BAR_DELIMITED_MSG_PT3, BAR_DELIMITED_MSG_PT4, "");
    private static final String NEWLINE = System.getProperty("line.separator");
    private static final Date START_DATE = DATE_20150911;

    private static final String[] EXPECTED_ADD_PRODUCT_MESSAGE_PARTS = new String[] {
        "",
        "MESSAGE^1^0",
        BAR_DELIMITED_MSG_PT1,
        "MESSAGE^2^0",
        BAR_DELIMITED_MSG_PT2,
        "MESSAGE^3^0",
        BAR_DELIMITED_MSG_PT3,
        "MESSAGE^4^0",
        BAR_DELIMITED_MSG_PT4,
        "MESSAGE^5^0",
        "",
        "MESSAGE^6^0",
        "    1 product name     [1 cmop id]",
        "MESSAGE^7^0",
        "    (DISPENSE UNIT - 1 dispense unit name)",
        "MESSAGE^8^0",
        "    1 ndc number",
        "MESSAGE^9^0",
        ""
    };

    private static final String[] EXPECTED_ADD_PRODUCT_MESSAGE_PARTS_2 = new String[] {
        "",
        "MESSAGE^1^0",
        BAR_DELIMITED_MSG_PT1,
        "MESSAGE^2^0",
        BAR_DELIMITED_MSG_PT2,
        "MESSAGE^3^0",
        BAR_DELIMITED_MSG_PT3,
        "MESSAGE^4^0",
        BAR_DELIMITED_MSG_PT4,
        "MESSAGE^5^0",
        "",
        "MESSAGE^6^0",
        "    1 product name     [1 cmop id]",
        "MESSAGE^7^0",
        "    (DISPENSE UNIT - 1 dispense unit name)",
        "MESSAGE^8^0",
        "    1 ndc number",
        "MESSAGE^9^0",
        "    2 product name     [2 cmop id]",
        "MESSAGE^10^0",
        "    (DISPENSE UNIT - 2 dispense unit name)",
        "MESSAGE^11^0",
        "    2 ndc number",
        "MESSAGE^12^0",
        "    3 product name     [3 cmop id]",
        "MESSAGE^13^0",
        "    (DISPENSE UNIT - 3 dispense unit name)",
        "MESSAGE^14^0",
        "    3 ndc number",
        "MESSAGE^15^0",
        ""
    };

    private static final String[] SAMPLE_MESSAGE_REPORT = new String[] {
            "MESSAGE^0^0",
            // Test subject does exactly this, unfortunately!
            "^^55^55^" + MumpsConverter.convertDate(new Date()),
            "MESSAGE^1^0", 
            "", 
            "MESSAGE^2^0", 
            "this is", 
            "MESSAGE^3^0",
            "a test", 
            "MESSAGE^4^0", 
            "of the", 
            "MESSAGE^5^0", 
            "whatever",
            "MESSAGE^6^0", 
            "", 
            "MESSAGE^7^0",
            "    productName_0     [cmopId_0]", 
            "MESSAGE^8^0",
            "    (DISPENSE UNIT - dispenseUnitName_0)", 
            "MESSAGE^9^0",
            "    ndcNumber_0", 
            "MESSAGE^10^0",
            "    productName_1     [cmopId_1]", 
            "MESSAGE^11^0",
            "    (DISPENSE UNIT - dispenseUnitName_1)", 
            "MESSAGE^12^0",
            "    ndcNumber_1", 
            "MESSAGE^13^0", 
            "", 
            "MESSAGE^14^0", 
            "this is",
            "MESSAGE^15^0", 
            "a test", 
            "MESSAGE^16^0", 
            "of the", 
            "MESSAGE^17^0",
            "whatever", 
            "MESSAGE^18^0", 
            "", 
            "MESSAGE^19^0",
            "    productName_0     [cmopId_0]", 
            "MESSAGE^20^0", 
            "",
            "MESSAGE^21^0", 
            "this is", 
            "MESSAGE^22^0", 
            "a test",
            "MESSAGE^23^0", 
            "of the", 
            "MESSAGE^24^0", 
            "whatever",
            "MESSAGE^25^0", 
            "", 
            "MESSAGE^26^0",
            "  FORMULARY ITEMS", 
            "MESSAGE^27^0", 
            "    NONE", 
            "MESSAGE^28^0",
            "", 
            "MESSAGE^29^0", 
            "  NON-FORMULARY ITEMS", 
            "MESSAGE^30^0",
            "    NONE", 
            "MESSAGE^31^0", 
            ""
    };

    static abstract class SampleData {
        protected abstract int getColumnCount();
        protected abstract String getColumnTitle(int col);
        private String getValue(String columnName, int rowNumber) {
            return columnName + "_" + rowNumber;
        }
        List<Object[]> getSample(int rows) {
            List<Object[]> result = new ArrayList<Object[]>();
            for (int row = 0; row < rows; row++) {
                result.add(getSampleFor(row));
            }
            return result;
        }
        String[] getSampleFor(int row) {
            String[] result = new String[getColumnCount()];
            for (int col = 0; col < getColumnCount(); col++) {
                String proposedValue = getValue(getColumnTitle(col), row);
                // can't pass our usual dummy data for a Date, so if the column is a date, just use this.
                if (proposedValue.contains("Date")) proposedValue = LONG_20150911 + "";
                result[col] = proposedValue;
            }
            return result;
        }
    }

    static class AddedProductsReportData extends SampleData {
        String[] colTitles = new String[] { "newValue", "productName",
                "cmopId", "dispenseUnitName", "ndcNumber" };

        @Override
        protected String getColumnTitle(int col) {
            return colTitles[col];
        }

        @Override
        protected int getColumnCount() {
            return 5;
        }
    }

    static class InactivatedCmopProductsData extends SampleData {
        String[] colTitles = new String[] { "newValue", "productName",
                "cmopId" };

        @Override
        protected String getColumnTitle(int col) {
            return colTitles[col];
        }

        @Override
        protected int getColumnCount() {
            return 3;
        }
    }

    static class FutureInactiveChangeData extends SampleData {
        String[] colTitles = new String[] { "eplId", "productName", "cmopId",
                "futureDate" };
        @Override
        protected String getColumnTitle(int col) {
            return colTitles[col];
        }

        @Override
        protected int getColumnCount() {
            return 4;
        }
    }

    private MessageDataCapabilityImpl testSubject;

    @Mock
    private EplNdfOutgoingDifferencesDao mockEplNdfOutgoingDifferencesDao;
    
    @Mock
    private UpdateFileReportGenerator mockUpdateFileReportGenerator;

    @Mock
    private NdfUpdateProcessFile mockNdfUpdateFile;

    @Mock
    private EplRematchDao mockEplRematchSuggestionDao;
    
    @Mock
    private EplProductDao mockEplProductDao;

    @Mock
    private SeekableByteChannel mockSeekableByteChannel;

    private static List<Object[]> getListOfOneAddedProduct() {
        List<Object[]> result = new ArrayList<Object[]>();
        result.add(getAddedProduct(1));
        return result;
    }

    private static List<Object[]> getListOfMultipleAddedProducts() {
        List<Object[]> result = new ArrayList<Object[]>();
        result.add(getAddedProduct(1));
        result.add(getAddedProduct(2));
        result.add(getAddedProduct(3));
        return result;
    }

    private static Object[] getAddedProduct(int num) {
        Object[] result = new Object[5];
        result[0] = num + " new value";
        result[1] = num + " product name";
        result[2] = num + " cmop id";
        result[3] = num + " dispense unit name";
        result[4] = num + " ndc number";
        return result;
    }

    @Before
    public void setup() {
        testSubject = new MessageDataCapabilityImpl(
            mockEplNdfOutgoingDifferencesDao,
            mockEplRematchSuggestionDao,
            mockEplProductDao,
            mockUpdateFileReportGenerator);
    }

    /*
     * temporal coupling in the method tested here--it must be called first as it takes the responsibility to start the message.
     */
    @Test
    public void should_start_message_when_no_added_products_are_present() throws Exception {
        // arrange
        StringBuffer message = new StringBuffer();
        when(mockEplNdfOutgoingDifferencesDao.findAddedProducts(VistaFileNumber.FIFTY_68.getFileNumber(), VISTA_FIELD_NUMBER_DOT_01, START_DATE)).thenReturn(null);

        // act
      /*  int counter = testSubject.addedProductReport(START_DATE);

        // assert (usually prefer to make just one simple assertion, but the class under test does way too much here)
        assertThat(counter, is(1));
        assertThat(message.toString(), is(NEWLINE + "MESSAGE^" + counter + "^0" + NEWLINE));*/
    }

    @Test
    public void should_start_message_and_add_products_when_one_added_product_is_present() throws Exception {
        // arrange
        StringBuffer message = new StringBuffer();
        when(mockEplNdfOutgoingDifferencesDao.findAddedProducts(VistaFileNumber.FIFTY_68.getFileNumber(), VISTA_FIELD_NUMBER_DOT_01, START_DATE)).thenReturn(getListOfOneAddedProduct());
        when(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.ADDEDPRODUCT))
          .thenReturn(SAMPLE_PADDED_HEADER_MESSAGE);

        // act
        /*int counter = testSubject.addedProductReport(START_DATE);

        // assert (usually prefer to make just one simple assertion, but the class under test does way too much here)
        assertThat(counter, is(9));
        String expectedReport = Arrays.stream(EXPECTED_ADD_PRODUCT_MESSAGE_PARTS).collect(Collectors.joining(NEWLINE));
        assertThat(message.toString(), is(expectedReport));*/
    }

    @Test
    public void should_start_message_and_add_products_when_multiple_added_products_are_present() throws Exception {
        // arrange
        StringBuffer message = new StringBuffer();
        when(mockEplNdfOutgoingDifferencesDao.findAddedProducts(VistaFileNumber.FIFTY_68.getFileNumber(), VISTA_FIELD_NUMBER_DOT_01, START_DATE)).thenReturn(getListOfMultipleAddedProducts());
        when(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.ADDEDPRODUCT))
          .thenReturn(SAMPLE_PADDED_HEADER_MESSAGE);

        // act
       /* int counter = testSubject.addedProductReport( START_DATE);

        // assert (usually prefer to make just one simple assertion, but the class under test does way too much here)
        assertThat(counter, is(15));
        String expectedReport = Arrays.stream(EXPECTED_ADD_PRODUCT_MESSAGE_PARTS_2).collect(Collectors.joining(NEWLINE));
        assertThat(message.toString(), is(expectedReport));*/
    }

    @Test
    @Ignore
    /**
     * This was written to capture the exact behavior of the 
     * messageReportData() method prior to refactoring. Intent is to
     * just use the refactored implementation now, so this is left as
     * a point of reference.
     * 
     * @throws Exception
     */
    public void should_generate_sample_message_report() throws Exception {
        when(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.NATIONALFORMULARY))
          .thenReturn(SAMPLE_PADDED_HEADER_MESSAGE);
        AddedProductsReportData addedProductsReportData = new AddedProductsReportData();
        when(mockEplNdfOutgoingDifferencesDao.findAddedProducts(VistaFileNumber.FIFTY_68.getFileNumber(), ".01", START_DATE)).thenReturn(addedProductsReportData.getSample(2));
        when(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.ADDEDPRODUCT))
          .thenReturn(SAMPLE_PADDED_HEADER_MESSAGE);
        InactivatedCmopProductsData inactivatedCmopProductsData = new InactivatedCmopProductsData();
        when(mockEplNdfOutgoingDifferencesDao.findInactivatedCMOPProducts(START_DATE)).thenReturn(inactivatedCmopProductsData.getSample(1));
        when(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.INACTIVATEDCMOP))
          .thenReturn(SAMPLE_PADDED_HEADER_MESSAGE);

        testSubject.buildDataUpdateForNdfReport(START_DATE);

        // These assertions are obsolete: they are from when MessageDataCapabilityImpl generated the update-file-formatted
        // report instead of the unformatted content.
        String expectedReport = Arrays.stream(SAMPLE_MESSAGE_REPORT)
                .collect(Collectors.joining(NEWLINE));
        verify(mockNdfUpdateFile).putNextRow(expectedReport);
    }
}
