package gov.va.med.pharmacy.peps.service.common.session.impl;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.easymock.EasyMock;
import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;
import org.springframework.transaction.PlatformTransactionManager;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.capability.ComplianceSiteDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.NationalSettingDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ReportDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.SiteListDomainCapability;
import gov.va.med.pharmacy.peps.service.common.session.DomainService;

/**
 * JUnit for ReportDataUpdateProcessImpl.
 */
public class ReportDataUpdateProcessImplTest extends EasyMockSupport {    

    /** The compliance site domain capability. */
    private ComplianceSiteDomainCapability complianceSiteDomainCapability;

    /** The site list domain capability. */
    private SiteListDomainCapability siteListDomainCapability;

    /** The report domain capability. */
    private ReportDomainCapability reportDomainCapability;

    /** The domain service. */
    private DomainService domainService;

    /** The transaction manager. */
    private PlatformTransactionManager transactionManager;

    /** The national setting domain capability. */
    private NationalSettingDomainCapability nationalSettingDomainCapability;

    /**
     * Setup before all tests.
     *
     * @throws Exception the exception
     */
    //
    @Before
    public void setUp() throws Exception {
        complianceSiteDomainCapability = mock(ComplianceSiteDomainCapability.class);
        siteListDomainCapability = mock(SiteListDomainCapability.class);
        reportDomainCapability = mock(ReportDomainCapability.class);
        domainService = mock(DomainService.class);
        transactionManager = mock(PlatformTransactionManager.class);
        nationalSettingDomainCapability = mock(NationalSettingDomainCapability.class);
    }

    /**
     * Test determineSitesForReport().
     * @throws Exception 
     */
    @Test
    public final void testDetermineSitesForReport() throws Exception {
        final int EXPECTED_RESULT_COUNT = 7;
        
        ComplianceSiteVo site1 = new ComplianceSiteVo(new VistaSiteVo("1", "SITE1 VAMC"), ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo site2 = new ComplianceSiteVo(new VistaSiteVo("2", "SITE2 VAMC"), ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo site3 = new ComplianceSiteVo(new VistaSiteVo("3", "SITE3 VAMC"), ComplianceSiteVo.EXCLUDE);
        ComplianceSiteVo site4 = new ComplianceSiteVo(new VistaSiteVo("4", "SITE4 VAMC"), ComplianceSiteVo.EXCLUDE);
        ComplianceSiteVo site5 = new ComplianceSiteVo(new VistaSiteVo("5", "SITE5 VAMC"), ComplianceSiteVo.INCLUDE);

        List<ComplianceSiteVo> included = Arrays.asList(site2, site1, site5);
        List<ComplianceSiteVo> excluded = Arrays.asList(site4, site3);

        VistaSiteVo vistaSite3 = new VistaSiteVo("3", "SITE3 VAMC");
        VistaSiteVo vistaSite4 = new VistaSiteVo("4", "SITE4 VAMC");
        VistaSiteVo vistaSite10 = new VistaSiteVo("10", "SITE10 VAMC");
        VistaSiteVo vistaSite11 = new VistaSiteVo("11", "SITE11 VAMC");
        VistaSiteVo vistaSite12 = new VistaSiteVo("12", "SITE12 VAMC");
        VistaSiteVo vistaSite13 = new VistaSiteVo("13", "SITE13 VAMC");

        List<VistaSiteVo> filteredList = Arrays.asList(vistaSite13, vistaSite12, vistaSite11, vistaSite10, vistaSite4,
            vistaSite3);

        EasyMock.expect(siteListDomainCapability.retrieveFiltered()).andReturn(filteredList);
        EasyMock.expect(complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE))
            .andReturn(included);
        EasyMock.expect(complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.EXCLUDE))
            .andReturn(excluded);
        replayAll();

        ReportDataUpdateProcessImpl impl = new ReportDataUpdateProcessImpl(complianceSiteDomainCapability,
            siteListDomainCapability, reportDomainCapability, domainService, transactionManager,
            nationalSettingDomainCapability);

        Set<VistaSiteVo> results = impl.determineSitesForReport();        
        assertThat(results.size(), equalTo(EXPECTED_RESULT_COUNT));
        String expectedSiteListCs = "SITE1 VAMC, SITE10 VAMC, SITE11 VAMC, SITE12 VAMC, SITE13 VAMC, SITE2 VAMC, SITE5 VAMC, ";
        String actualSiteListCs = new String();

        for (VistaSiteVo vistaSiteVo : results) {
            actualSiteListCs = actualSiteListCs + vistaSiteVo.getName() + ", ";
        }

        assertThat(actualSiteListCs, equalTo(expectedSiteListCs));
    }

}
