package gov.va.med.pharmacy.peps.presentation.common.utility;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Locale;

import org.junit.After;
import org.junit.Test;
import org.springframework.context.support.StaticMessageSource;

import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;

public class JspTagUtilityTest {

    @Test
    public void testGetTooltip() {
        /* Setup */
        final StaticMessageSource messageSource = new StaticMessageSource();
        messageSource.addMessage(JspTagUtility.SELECT_TOOLTIP_MESSAGE_CODE, Locale.getDefault(), "Test {0} message");
        final JspTagUtility tagUtility = new JspTagUtility();
        tagUtility.setMessageSource(messageSource);
        final ProductVo sampleProduct = new ProductGenerator().getFirst();

        /* Behavior & Verification */
        assertThat(
            "typical FieldKey case - not even a ValueObject",
            JspTagUtility.getTooltip(Locale.getDefault(), FieldKey.CMOP_DISPENSE, null, Integer.valueOf(5)),
            equalTo("CMOP Dispense (National)"));
        assertThat(
            "SELECT case - ValueObject",
            JspTagUtility.getTooltip(Locale.getDefault(), FieldKey.SELECT, sampleProduct.getEntityType(), sampleProduct),
            equalTo("Test " + sampleProduct.toShortString() + " message"));
        assertThat(
            "SELECT case - not a ValueObject",
            JspTagUtility.getTooltip(Locale.getDefault(), FieldKey.SELECT, null, Integer.valueOf(5)),
            equalTo("Test  message"));
    }
    
    @After
    public void tearDown() {
        // Clear out the static field to make sure one test doesn't affect another.
        final JspTagUtility tagUtility = new JspTagUtility();
        tagUtility.setMessageSource(null);
    }

}
